@@ -31,11 +31,12 @@
          (won nil)
          (asmstream (make-asmstream))
          (*asmstream* asmstream)
-         (*adjustable-vectors* nil))
+         (sb-vm::*adjustable-vectors* nil))
     (unwind-protect
         (let ((sb-xc:*features* (cons :sb-assembling sb-xc:*features*))
               (*readtable* sb-cold:*xc-readtable*))
           (load (merge-pathnames name (make-pathname :type "lisp")))
+          (resolve-ep-labels (asmstream-code-section asmstream))
           ;; Reserve space for the jump table. The first word of the table
           ;; indicates the total length in words, counting the length word itself
           ;; as a word, and the words comprising jump tables that were registered
@@ -48,6 +49,7 @@
                 (n-data-words  ; Space for internal jump tables if needed
                  (loop for ((category . data) . label)
                        across (asmstream-constant-vector asmstream)
+                       do (progn #+host-quirks-ccl label) ; shut up the host
                        when (eq category :jump-table) sum (length data))))
             (emit (asmstream-data-section asmstream)
                   `(.lispword ,(+ n-extra-words n-data-words 1))
@@ -68,6 +70,7 @@
             (dump-assembler-routines segment
                                      (segment-buffer segment)
                                      (sb-assem::segment-fixup-notes segment)
+                                     (sb-assem::get-allocation-points asmstream)
                                      *entry-points*
                                      lap-fasl-output))
           (setq won t))
@@ -120,6 +123,26 @@
     for export in exports
     collect `(push ,(expand-one-export-spec export) *entry-points*)))
 
+(defun resolve-ep-labels (section)
+  (do ((statement (stmt-next (section-start section)) (stmt-next statement)))
+      ((null statement))
+    ;; We also output some raw words using fixups; I'm not sure if they should be
+    ;; changed to labels. For now, restrict to control transfers.
+    (binding* ((patch
+                (when (member (stmt-mnemonic statement)
+                              '("B" "BEQ" "JMP" "CALL") ; KLUDGE
+                              :test 'string=)
+                  (member-if (lambda (x)
+                               (and (typep x 'fixup)
+                                    (eq (fixup-flavor x) :assembly-routine)
+                                    (eql (fixup-offset x) 0)))
+                             (stmt-operands statement)))
+                :exit-if-null)
+               (ep (assoc (fixup-name (car patch)) *entry-points*)))
+      ;; oy. what is (third ep) ? An offset?
+      (aver (and ep (= (third ep) 0)))
+      (rplaca patch (second ep)))))
+
 (defun expand-align-option (align)
   (when align
     `((emit-alignment ,align))))
@@ -226,15 +249,9 @@
                       :key #'car)
          (:generator ,cost
            ,@(mapcar (lambda (arg)
-                       #+(or hppa alpha) `(move ,(reg-spec-name arg)
-                                                 ,(reg-spec-temp arg))
-                       #-(or hppa alpha) `(move ,(reg-spec-temp arg)
-                                                 ,(reg-spec-name arg)))
+                       `(move ,(reg-spec-temp arg) ,(reg-spec-name arg)))
                      args)
            ,@call-sequence
            ,@(mapcar (lambda (res)
-                       #+(or hppa alpha) `(move ,(reg-spec-temp res)
-                                                 ,(reg-spec-name res))
-                       #-(or hppa alpha) `(move ,(reg-spec-name res)
-                                                 ,(reg-spec-temp res)))
+                       `(move ,(reg-spec-name res) ,(reg-spec-temp res)))
                      results))))))
