@@ -5,51 +5,65 @@
 
 (in-package "SB-VM")
 
-(macrolet ((do-fprs (operation regset)
+(macrolet ((do-fprs (operation regset &aux (displacement 0))
+             ;; The YMM case could be removed now I suppose, since we use XSAVE + XRSTOR
              (multiple-value-bind (mnemonic fpr-align)
                  (ecase regset
                    (:xmm (values 'movaps 16))
                    (:ymm (values 'vmovaps 32)))
-               `(progn
-                  ,@(loop for regno below 16
-                       collect
-                         (ecase operation
-                           (push
-                            `(inst ,mnemonic (ea ,(+ 8 (* regno fpr-align)) rsp-tn)
-                                   (sb-x86-64-asm::get-fpr ,regset ,regno)))
-                           (pop
-                            `(inst ,mnemonic (sb-x86-64-asm::get-fpr ,regset ,regno)
-                                   (ea ,(+ 8 (* regno fpr-align)) rsp-tn)))))))))
+               (collect ((insts))
+                 ;; RAX as the base register encodes shorter than RSP in an EA.
+                 (insts '(inst lea rax-tn (ea 8 rsp-tn)))
+                 (dotimes (regno 16 `(progn ,@(insts)))
+                   (when (>= displacement 128)
+                     (insts '(inst add rax-tn 128))
+                     (decf displacement 128)) ; EA displacement stays 1-byte this way
+                   (let ((fpr `(sb-x86-64-asm::get-fpr ,regset ,regno)))
+                     (insts (ecase operation
+                              (push `(inst ,mnemonic (ea ,displacement rax-tn) ,fpr))
+                              (pop `(inst ,mnemonic ,fpr (ea ,displacement rax-tn))))))
+                   (incf displacement fpr-align))))))
   ;; Caller will have allocated 512+64+256 bytes above the stack-pointer
   ;; prior to the CALL. Use that as the save area.
-  (define-assembly-routine (save-ymm) ()
-    (inst push rax-tn)
+  (define-assembly-routine (fpr-save) ()
+    (test-cpu-feature cpu-has-ymm-registers)
+    (inst jmp :nz have-ymm)
+    (do-fprs push :xmm)
+    (inst ret)
+    HAVE-YMM
+    ;; Although most of the time RDX can be clobbered, some of the time it can't.
+    ;; If WITH-REGISTERS-PRESERVED wraps a lisp function to make it appear to preserve
+    ;; all registers, we obviously need to return its primary value in RDX.
+    ;; RAX need not be saved though.
     (inst push rdx-tn)
-    (inst mov rax-tn 7)
     (zeroize rdx-tn)
+    ;; After PUSH the save area is at RSP+16 with the return-PC at [RSP+8]
     ;; Zero the header
-    (loop for i from (+ 512 24) by 8
-          repeat 8
-          do
-          (inst mov (ea i rsp-tn) rdx-tn))
-    (inst xsave (ea 24 rsp-tn))
-    (inst pop rdx-tn)
-    (inst pop rax-tn))
-  (define-assembly-routine (restore-ymm) ()
-    (inst push rax-tn)
-    (inst push rdx-tn)
+    (inst lea rax-tn (ea (+ 512 16) rsp-tn))
+    (dotimes (i 8)
+      (inst mov (ea (ash i word-shift) rax-tn) rdx-tn))
     (inst mov rax-tn 7)
+    (inst xsave (ea 16 rsp-tn))
+    (inst pop rdx-tn))
+
+  (define-assembly-routine (fpr-restore) ()
+    (test-cpu-feature cpu-has-ymm-registers) (inst jmp :nz have-ymm)
+    (do-fprs pop :xmm)
+    (inst ret)
+    HAVE-YMM
+    (inst push rdx-tn)
+    (inst mov rax-tn 7) ; OK to clobber RAX
     (zeroize rdx-tn)
-    (inst xrstor (ea 24 rsp-tn))
-    (inst pop rdx-tn)
-    (inst pop rax-tn))
-  ;; As above, but only 256 bytes of the save area are needed, the rest goes to waste.
-  (define-assembly-routine (save-xmm (:export fpr-save)) ()
-    fpr-save ; KLUDGE: this is element 4 of the entry point vector
-    (do-fprs push :xmm))
-  (define-assembly-routine (restore-xmm (:export fpr-restore)) ()
-    fpr-restore ; KLUDGE: this is element 6 of the entry point vector
-    (do-fprs pop :xmm)))
+    (inst xrstor (ea 16 rsp-tn))
+    (inst pop rdx-tn)))
+
+(define-assembly-routine (switch-to-arena (:return-style :raw)) ()
+  (inst mov rsi-tn (ea rsp-tn)) ; explicitly  pass the return PC
+  ;; RSI and RDI are vop temps, so don't bother preserving them
+  (with-registers-preserved (c :except (rsi rdi))
+    (pseudo-atomic ()
+      #-system-tlabs (inst break halt-trap)
+      #+system-tlabs (inst call (make-fixup "switch_to_arena" :foreign)))))
 
 (macrolet ((def-routine-pair (name&options vars &body code)
              `(progn
@@ -131,27 +145,39 @@
 (define-assembly-routine
     (undefined-alien-tramp (:return-style :none))
     ()
+  ;; This routine computes into RBX the address of the linkage table entry that was called,
+  ;; corresponding to the undefined alien function.
   (inst push rax-tn) ; save registers in case we want to see the old values
   (inst push rbx-tn)
   ;; load RAX with the PC after the call site
   (inst mov rax-tn (ea 16 rsp-tn))
   ;; load RBX with the signed 32-bit immediate from the call instruction
   (inst movsx '(:dword :qword) rbx-tn (ea -4 rax-tn))
+  ;; The decoding seems scary, but it's actually not. Any C call-out instruction has
+  ;; a 4-byte trailing operand, with the preceding byte being unique.
   ;; if at [PC-5] we see #x25 then it was a call with 32-bit mem addr
   ;; if ...              #xE8 then ...                32-bit offset
-  (inst cmp :byte (ea -5 rax-tn) #x25)
-  (inst jmp :e ABSOLUTE)
-  (inst cmp :byte (ea -5 rax-tn) #xE8)
-  (inst jmp :e RELATIVE)
+  ;; if ...              #x92 then it was "call *DISP(%r10)" where r10 is the table base
+  #-immobile-space ; only non-relocatable alien linkage table can use "CALL [ABS]" form
+  (progn (inst cmp :byte (ea -5 rax-tn) #x25)
+         (inst jmp :e ABSOLUTE))
+  #+immobile-space ; only relocatable alien linkage table can use "CALL rel32" form
+  (progn (inst cmp :byte (ea -5 rax-tn) #xE8)
+         (inst jmp :e RELATIVE)
+         (inst cmp :byte (ea -5 rax-tn) #x92)
+         (inst jmp :e ABSOLUTE))
   ;; failing those, assume RBX was valid. ("can't happen")
   (inst mov rbx-tn (ea rsp-tn)) ; restore pushed value of RBX
   (inst jmp trap)
   ABSOLUTE
-  (inst lea rbx-tn (ea -8 rbx-tn))
+  #-immobile-space (inst sub rbx-tn 8)
+  #+immobile-space (inst lea rbx-tn (ea -8 r10-tn rbx-tn))
   (inst jmp TRAP)
   RELATIVE
   (inst add rbx-tn rax-tn)
   TRAP
+  ;; XXX: why aren't we adding something to the stack pointer to balance the two pushes?
+  ;; (I guess we can only THROW at this point, so it doesn't matter)
   (error-call nil 'undefined-alien-fun-error rbx-tn))
 
 ;;; the closure trampoline - entered when a global function is a closure
@@ -173,6 +199,7 @@
   (loadw rax-tn rax-tn fdefn-fun-slot other-pointer-lowtag)
   (inst jmp (object-slot-ea rax-tn closure-fun-slot fun-pointer-lowtag)))
 
+#-compact-instance-header
 (define-assembly-routine
     (funcallable-instance-tramp (:return-style :none))
     ()
