@@ -47,10 +47,6 @@
 
 (defun !c-runtime-noinform-p () (/= (extern-alien "lisp_startup_options" char) 0))
 
-(defun !format-cold-init ()
-  (sb-format::!late-format-init)
-  (sb-format::!format-directives-init))
-
 ;;; Allows the SIGNAL function to be called early.
 (defun !signal-function-cold-init ()
   #+sb-devel
@@ -84,10 +80,11 @@
           #+win32 (sb-win32::get-std-handle-or-null sb-win32::+std-error-handle+))
         (buf (make-string 1 :element-type 'base-char :initial-element #\Space)))
     (%make-fd-stream
-     :out (lambda (stream ch)
-            (declare (ignore stream))
-            (setf (char buf 0) ch)
-            (sb-unix:unix-write stderr buf 0 1))
+     :cout (lambda (stream ch)
+             (declare (ignore stream))
+             (setf (char buf 0) ch)
+             (sb-unix:unix-write stderr buf 0 1)
+             ch)
      :sout (lambda (stream string start end)
              (declare (ignore stream))
              (flet ((out (s start len)
@@ -106,7 +103,7 @@
                (:charpos ; impart just enough smarts to make FRESH-LINE dtrt
                 (if (eql (char buf 0) #\newline) 0 1)))))))
 
-(defun xc-sanity-checks ()
+(defun !xc-sanity-checks ()
   ;; Verify on startup that some constants were dumped reflecting the
   ;; correct action of our vanilla-host-compatible functions.  For
   ;; now, just SXHASH is checked.
@@ -115,21 +112,18 @@
   ;; side effect of data recording when invoking compile-time
   ;; functions aren't propagated back to process that forked the
   ;; children doing the grunt work.
-  (let ((sxhash-crosscheck
-          '#.(let (pairs)
-               ;; De-duplicate, which reduces the list from ~8000 entries to ~1000 entries.
-               ;; But make sure that any key which is repeated has the same value
-               ;; at each repetition.
-               (dolist (pair *sxhash-crosscheck* (coerce pairs 'simple-vector))
-                 (let ((found (assoc (car pair) pairs)))
-                   (if found
-                       (aver (= (cdr found) (cdr pair)))
-                       (push pair pairs)))))))
-    (loop for (object . hash) across sxhash-crosscheck
-          unless (= (sxhash object) hash)
-            do (error "SXHASH computed wrong answer for ~S. Got ~x should be ~x"
-                      object hash (sxhash object))))
-  (format t "~&cross-compiler SXHASH tests passed~%"))
+  (let ((list (with-open-file (stream "output/sxhash-calls.lisp-expr"
+                                      :if-does-not-exist nil)
+                (when stream
+                  (let ((*package* (find-package "SB-KERNEL"))) (read stream))))))
+    (when list
+      (dolist (item list)
+        (destructuring-bind (object hash) item
+          (unless (= (sxhash object) hash)
+            (error "SXHASH computed wrong answer for ~S. Got ~x should be ~x"
+                   object hash (sxhash object)))))
+      (format t "~&cross-compiler SXHASH tests passed: ~D cases~%"
+              (length list)))))
 
 ;;; called when a cold system starts up
 (defun !cold-init ()
@@ -147,7 +141,6 @@
           *trace-output* stream))
   (show-and-call !signal-function-cold-init)
   (show-and-call !printer-control-init) ; needed before first instance of FORMAT or WRITE-STRING
-  (setq *unparse-fun-type-simplify* nil) ; needed by TLFs in target-error.lisp
   (setq sb-unix::*unblock-deferrables-on-enabling-interrupts-p* nil) ; needed by LOAD-LAYOUT called by CLASSES-INIT
   (setq *print-length* 6
         *print-level* 3)
@@ -239,9 +232,15 @@
                (funcall (second toplevel-thing))))
         ((cons (eql :load-time-value-fixup))
          (destructuring-bind (object index value) (cdr toplevel-thing)
-           (aver (typep object 'code-component))
-           (aver (unbound-marker-p (code-header-ref object index)))
-           (setf (code-header-ref object index) (svref *!load-time-values* value))))
+           (let ((replacement (svref *!load-time-values* value)))
+             (etypecase object
+               (code-component
+                (aver (unbound-marker-p (code-header-ref object index)))
+                (setf (code-header-ref object index) replacement))
+               (cons
+                (aver (= index 0))
+                (aver (unbound-marker-p (car object)))
+                (rplaca object replacement))))))
         ((cons (eql :named-constant))
          (destructuring-bind (object index name) (cdr toplevel-thing)
            (aver (typep object 'code-component))
@@ -326,7 +325,7 @@
       (logically-readonlyize (sb-c::sc-move-vops sc))
       (logically-readonlyize (sb-c::sc-move-costs sc))))
 
-  (show-and-call xc-sanity-checks)
+  (show-and-call !xc-sanity-checks)
 
   ;; The system is finally ready for GC.
   (/show0 "enabling GC")
@@ -406,8 +405,6 @@ process to continue normally."
       (sb-thread::init-main-thread)
       #+x86-64 (sb-vm::validate-asm-routine-vector)
       (rebuild-package-vector))
-    ;; Initializing the standard streams calls ALLOC-BUFFER which calls FINALIZE
-    (finalizers-reinit)
     ;; Initialize streams next, so that any errors can be printed
     (stream-reinit t)
     (rebuild-pathname-cache)
