@@ -21,7 +21,7 @@
   (let ((lambda (if (typep expr '(cons (eql function) (cons t null)))
                     (cadr expr)
                     expr)))
-    (if (typep lambda '(cons (member lambda named-lambda lambda-with-lexenv)))
+    (if (typep lambda '(cons (member lambda named-lambda)))
         (values lambda nil)
         (values `(lambda ()
                  ;; why PROGN? So that attempts to eval free declarations
@@ -100,7 +100,8 @@
            ;; 2002-10-24
            (let* ((sb-c:*lexenv* lexenv)
                   (sb-c::*ir1-namespace* (sb-c::make-ir1-namespace))
-                  (sb-c::*undefined-warnings* nil))
+                  (sb-c::*undefined-warnings* nil)
+                  sb-c::*argument-mismatch-warnings*)
              ;; FIXME: VALUES declaration
              (sb-c::process-decls decls
                                   vars
