@@ -51,11 +51,9 @@
   (setq *gc-inhibit* nil)
   ;; This GC looks utterly unnecessary, but when I tried removing it,
   ;; I got heap exhaustions sooner than when I left it in.
-  ;; So why is that? Well, it seems like there's some state that is getting
-  ;; saved into the core which needs resetting, such as these SETQs,
-  ;; but how is that affecting the C runtime's decision on when to
-  ;; perform the first auto-triggered GC? How is not doing an explicit
-  ;; GC delaying the decision? It seems wrong.
+  ;; I think the reason is that we never set the auto-trigger at all
+  ;; unless a GC happens. Maybe what we mean here is "set auto trigger"
+  ;; and we need a function in C to do that.
   (gc)
   (setf *n-bytes-freed-or-purified* 0
         *gc-run-time* 0))
@@ -133,6 +131,7 @@ run in any thread.")
          nil)
         (t
          (flet ((perform-gc ()
+                  (declare (sb-c::tlab :system)) ; for *gc-epoch*
                   ;; Called from WITHOUT-GCING and WITHOUT-INTERRUPTS
                   ;; after the world has been stopped, but it's an
                   ;; awkwardly long piece of code to nest so deeply.
@@ -223,7 +222,8 @@ run in any thread.")
   ;; which is an arbitrary one. If those actions aquire any locks, or are sensitive
   ;; to the state of *ALLOW-WITH-INTERRUPTS*, any deadlocks of what-have-you
   ;; are user error. Hooks need to be sufficiently uncomplicated as to be harmless.
-  (call-hooks "after-GC" *after-gc-hooks* :on-error :warn))
+  (sb-vm:without-arena "post-gc"
+    (call-hooks "after-GC" *after-gc-hooks* :on-error :warn)))
 
 #-sb-thread
 (defun post-gc ()
@@ -271,8 +271,7 @@ run in any thread.")
     ;; interrupts, which is precisely the thing we need to NOT do if already
     ;; in post-GC code of any kind (be it finalizer or other).
   (when (and *allow-with-interrupts*
-               (or (and (sb-impl::hash-table-culled-values
-                         (sb-impl::finalizer-id-map sb-impl::**finalizer-store**))
+               (or (and sb-impl::*finalizers-triggered*
                         (not sb-impl::*in-a-finalizer*))
                    *after-gc-hooks*))
       (sb-thread::without-thread-waiting-for ()
@@ -326,8 +325,7 @@ guaranteed to be collected."
          ;; but there is no automatic cache rehashing after GC.
          (sb-format::tokenize-control-string-cache-clear))
         ((eql 1 gen)
-         (sb-format::tokenize-control-string-cache-clear)
-         (ctype-of-cache-clear))
+         (sb-format::tokenize-control-string-cache-clear))
         (t
          (drop-all-hash-caches)))
   #-gencgc
