@@ -155,25 +155,6 @@
                (ash header-word (- hash-slot-present-flag))
                1))))
 
-;;; Macros not needed after this file (and avoids a redefinition warning this way)
-(eval-when (:compile-toplevel)
-(defmacro widetag@baseptr (sap)
-  #+big-endian `(sap-ref-8 ,sap ,(1- n-word-bytes))
-  #+little-endian `(sap-ref-8 ,sap 0))
-
-(defmacro lispobj@baseptr (sap widetag)
-  `(%make-lisp-obj
-    (logior (sap-int ,sap)
-            (logand (deref (extern-alien "widetag_lowtag" (array char 256)) ,widetag)
-                    lowtag-mask)))))
-
-;;; This uses the funny fixnum representation of ADDRESS. I'd like to change this
-;;; to take a SAP but god forbid people are using it?
-;;; DO NOT USE THIS! It is soon to be removed
-(defun reconstitute-object (address)
-  (let ((sap (descriptor-sap address)))
-    (lispobj@baseptr sap (widetag@baseptr sap))))
-
 ;;; Iterate over all the objects in the contiguous block of memory
 ;;; with the low address at START and the high address just before
 ;;; END, calling FUN with the object, the object's type code, and the
@@ -753,7 +734,7 @@ We could try a few things to mitigate this:
                    (#.code-header-widetag
                     (let ((dinfo (%code-debug-info obj)))
                       (format stream "~&Code object: ~S~%"
-                              (if dinfo
+                              (if dinfo ; BUG: what if this is in the asm code ?
                                   (sb-c::compiled-debug-info-name dinfo)
                                   "No debug info."))))
                    (#.symbol-widetag
@@ -1025,6 +1006,27 @@ We could try a few things to mitigate this:
      win
        (return-from references-p t))))
 
+;;; If OBJ points (directly or indirectly) to something in some arena,
+;;; then return the pointed-to arena-allocated thing.
+;;; Cribbed from DEEP-SIZE in tests/do-refs.impure
+#+system-tlabs
+(defun points-to-arena (obj)
+  (flet ((leafp (x) (typep x '(or package symbol fdefn wrapper classoid))))
+    (let ((worklist (list obj))
+          (seen (make-hash-table :test 'eq)))
+      (setf (gethash obj seen) t)
+      (flet ((visit (thing)
+               (when (is-lisp-pointer (get-lisp-obj-address thing))
+                 (unless (or (leafp thing) (gethash thing seen))
+                   (when (find-containing-arena (get-lisp-obj-address thing))
+                     (return-from points-to-arena thing))
+                   (push thing worklist)
+                   (setf (gethash thing seen) t)))))
+        (loop
+          (unless worklist (return))
+          (let ((x (pop worklist)))
+            (do-referenced-object (x visit))))))))
+
 ;;; This interface allows one either to be agnostic of the referencing space,
 ;;; or specify exactly one space, but not specify a list of spaces.
 ;;; An upward-compatible change would be to assume a list, and call ENSURE-LIST.
@@ -1415,9 +1417,11 @@ We could try a few things to mitigate this:
                      (type-of x)
                      (type-of pointee)))))))
 
+(macrolet ((aligned-base (blk)
+             `(align-up (sap-int (sap+ ,blk (* 4 n-word-bytes))) 4096)))
 (defun dump-arena-objects (arena &aux (tot-size 0))
   (do-arena-blocks (memblk arena)
-    (let ((from (sap-int memblk) )
+    (let ((from (aligned-base memblk))
           (to (sap-int (arena-memblk-freeptr memblk))))
       (format t "~&Memory block ~X..~X~%" from to)
       (map-objects-in-range
@@ -1425,10 +1429,33 @@ We could try a few things to mitigate this:
          (declare (ignore type))
          (incf tot-size size)
          (format t "~x ~s~%" (get-lisp-obj-address obj) (type-of obj)))
-       (make-lisp-obj from)
-       (make-lisp-obj to))))
+       (%make-lisp-obj from)
+       (%make-lisp-obj to))))
   tot-size)
-)
+(defun arena-contents (arena)
+  (let ((count 0))
+    (do-arena-blocks (memblk arena)
+      (let ((base (aligned-base memblk))
+            (limit (sap-int (arena-memblk-freeptr memblk))))
+        (map-objects-in-range
+         (lambda (obj widetag size)
+           (declare (ignore obj widetag size))
+           (incf count))
+         (%make-lisp-obj base)
+         (%make-lisp-obj limit))))
+    (let ((result (make-array count))
+          (index 0))
+      (do-arena-blocks (memblk arena)
+        (let ((base (aligned-base memblk))
+              (limit (sap-int (arena-memblk-freeptr memblk))))
+          (map-objects-in-range
+           (lambda (obj widetag size)
+             (declare (ignore widetag size))
+             (setf (aref result index) obj)
+             (incf count))
+           (%make-lisp-obj base)
+           (%make-lisp-obj limit))))
+      result)))))
 
 (in-package "SB-C")
 ;;; As soon as practical in warm build it makes sense to add
