@@ -750,11 +750,13 @@
          (or (search "src/pcl" stem)
              (search "src/code/alieneval" stem)
              (search "src/code/arena" stem)
-             (search "src/code/debug" stem)
-             (search "src/code/early-defmethod" stem)
-             (search "src/code/format" stem)
+             (search "src/code/avltree" stem)
              (search "src/code/brothertree" stem)
-             (search "src/code/avltree" stem))))
+             (search "src/code/debug" stem) ; also matches debug-{info,int,var-io}
+             (search "src/code/early-defmethod" stem)
+             (search "src/code/final" stem)
+             (search "src/code/format" stem)
+             (search "src/code/solist" stem))))
     (funcall *in-target-compilation-mode-fn*
            (lambda ()
              (progv (list (intern "*SOURCE-NAMESTRING*" "SB-C")
