@@ -69,6 +69,17 @@
   (values))
 
 
+(defun node-asserted-type (node)
+  (let ((dtype (node-derived-type node)))
+    (or
+     (binding* ((lvar (node-lvar node) :exit-if-null)
+                (dest (lvar-dest lvar)))
+       (when (and (cast-p dest)
+                  (eq (cast-type-to-check dest) *wild-type*))
+         (values-type-intersection
+          dtype (cast-asserted-type dest))))
+     dtype)))
+
 ;;;; stuff for checking a call against a function type
 
 ;;; Determine whether a use of a function is consistent with its type.
@@ -117,28 +128,20 @@
                (caller (loop for annotation in (lvar-annotations fun)
                              when (typep annotation 'lvar-function-designator-annotation)
                              do (setf *compiler-error-context* annotation)
-                                (return (lvar-function-designator-annotation-caller annotation)))))
+                                (return (lvar-function-designator-annotation-caller annotation))))
+               (name (nth-value 1 (lvar-fun-type fun))))
           (cond
-            ((report-arg-count-mismatch (nth-value 1 (lvar-fun-type fun))
+            ((report-arg-count-mismatch name
                                         caller type nargs nil
                                         :lossage-fun #'note-lossage))
             (t
              (check-fixed-and-rest args (append required optional) rest)
              (when (and keyp
-                        (check-key-args args max-args type))
+                        (check-key-args name args max-args type))
                (setf unknown-keys t))))
 
           (when result-test
-            (let* ((dtype (node-derived-type call))
-                   (out-type (or
-                              (binding* ((lvar (node-lvar call) :exit-if-null)
-                                         (dest (lvar-dest lvar)))
-                                (when (and (cast-p dest)
-                                           (eq (cast-type-to-check dest) *wild-type*)
-                                           (immediately-used-p lvar call))
-                                  (values-type-intersection
-                                   dtype (cast-asserted-type dest))))
-                              dtype))
+            (let* ((out-type (node-asserted-type call))
                    (return-type (fun-type-returns type)))
               (multiple-value-bind (int win) (funcall result-test out-type return-type)
                 (cond ((not win)
@@ -239,7 +242,7 @@ and no value was provided for it." name))))))))))
      (let* ((ctype (lvar-type lvar))
             (int (funcall *ctype-test-fun* ctype type)))
        (cond ((not int)
-              (unless (type= ctype (specifier-type '(member dummy)))
+              (unless (type= ctype (specifier-type '(eql dummy)))
                 (note-lossage "The ~:R argument is a ~S, not a ~S."
                               n (type-specifier ctype) (type-specifier type)))
               nil)
@@ -268,8 +271,8 @@ and no value was provided for it." name))))))))))
 ;;; Check that each of the type of each supplied argument intersects
 ;;; with the type specified for that argument. If we can't tell, then
 ;;; we can complain about the absence of manifest winnage.
-(declaim (ftype (function (list list (or ctype null)) (values)) check-fixed-and-rest))
 (defun check-fixed-and-rest (args types rest)
+  (declare (list args types) (type (or ctype null) rest))
   (do ((arg args (cdr arg))
        (type types (cdr type))
        (n 1 (1+ n)))
@@ -286,8 +289,9 @@ and no value was provided for it." name))))))))))
 ;;; be known and the corresponding argument should be of the correct
 ;;; type. If the key isn't a constant, then we can't tell, so we can
 ;;; complain about absence of manifest winnage.
-(declaim (ftype (function (list fixnum fun-type) boolean) check-key-args))
-(defun check-key-args (args pre-key type)
+(defun check-key-args (name args pre-key type)
+  (declare (list args) (fixnum pre-key) (type fun-type type))
+  (declare (ignorable name))
   (let (lossages allow-other-keys
         unknown-keys)
     (do ((key (nthcdr pre-key args) (cddr key))
@@ -325,20 +329,27 @@ and no value was provided for it." name))))))))))
                                     (1+ n)))))))))
     (when (and lossages (member allow-other-keys '(nil :no)))
       (setf lossages (nreverse lossages))
-      (if (cdr lossages)
-          (note-lossage "~@<~{~S~^, ~} and ~S are not a known argument keywords.~:@>"
-                        (butlast lossages)
-                        (car (last lossages)))
-          (note-lossage "~S is not a known argument keyword."
-                        (car lossages))))
+
+      (cond #-sb-xc-host
+            ((or (eq (info :function :type name) :generic-function)
+                 (eq (info :function :where-from name) :defined-method))
+             (note-key-arg-mismatch name lossages))
+            ((cdr lossages)
+             (note-lossage "~@<~{~S~^, ~} and ~S are not a known argument keywords.~:@>"
+                           (butlast lossages)
+                           (car (last lossages))))
+            (t
+             (note-lossage "~S is not a known argument keyword."
+                           (car lossages)))))
     unknown-keys))
 
 ;;; Construct a function type from a definition.
 ;;;
 ;;; Due to the lack of a (LIST X) type specifier, we can't reconstruct
 ;;; the &REST type.
-(declaim (ftype (sfunction (functional) fun-type) definition-type))
 (defun definition-type (functional)
+  (declare (type functional functional)
+           #-sb-xc-host (values fun-type))
   (if (lambda-p functional)
       (make-fun-type
        :required (mapcar #'leaf-type (lambda-vars functional))
@@ -357,8 +368,7 @@ and no value was provided for it." name))))))))))
                     (:required (req type))
                     (:optional (opt type))
                     (:keyword
-                     (keys (make-key-info :name (arg-info-key info)
-                                          :type type)))
+                     (keys (make-key-info (arg-info-key info) type)))
                     ((:rest :more-context)
                      (setq rest *universal-type*))
                     (:more-count))
@@ -368,7 +378,7 @@ and no value was provided for it." name))))))))))
            :required (req)
            :optional (opt)
            :rest rest
-           :keywords (keys)
+           :keywords (sb-kernel::intern-key-infos (keys))
            :keyp (optional-dispatch-keyp functional)
            :allowp (optional-dispatch-allowp functional)
            :returns (let ((tail-set (lambda-tail-set
@@ -423,11 +433,11 @@ and no value was provided for it." name))))))))))
 ;;; Return an APPROXIMATE-FUN-TYPE representing the context of
 ;;; CALL. If TYPE is supplied and not null, then we merge the
 ;;; information into the information already accumulated in TYPE.
-(declaim (ftype (function (combination
-                           &optional (or approximate-fun-type null))
-                          approximate-fun-type)
-                note-fun-use))
 (defun note-fun-use (call &optional type)
+  (declare (type combination call)
+           (type (or approximate-fun-type null) type)
+           #-sb-xc-host
+           (values approximate-fun-type))
   (let* ((type (or type (make-approximate-fun-type)))
          (types (approximate-fun-type-types type))
          (args (combination-args call))
@@ -488,16 +498,17 @@ and no value was provided for it." name))))))))))
 
 ;;; This is similar to VALID-FUN-USE, but checks an
 ;;; APPROXIMATE-FUN-TYPE against a real function type.
-(declaim (ftype (function (approximate-fun-type fun-type
-                           &optional function function function)
-                          (values boolean boolean))
-                valid-approximate-type))
 (defun valid-approximate-type (call-type type &optional
                                          (*ctype-test-fun*
                                           #'types-equal-or-intersect)
                                          (*lossage-fun*
                                           #'compiler-style-warn)
                                          (*unwinnage-fun* #'compiler-notify))
+  (declare (type approximate-fun-type call-type)
+           (type fun-type type)
+           (function *ctype-test-fun* *lossage-fun* *unwinnage-fun*)
+           #-sb-xc-host
+           (values boolean boolean))
   (let* ((*lossage-detected* nil)
          (*unwinnage-detected* nil)
          (required (fun-type-required type))
@@ -541,10 +552,10 @@ and no value was provided for it." name))))))))))
 
 ;;; Check that each of the types used at each arg position is
 ;;; compatible with the actual type.
-(declaim (ftype (function (approximate-fun-type list (or ctype null))
-                          (values))
-                check-approximate-fixed-and-rest))
 (defun check-approximate-fixed-and-rest (call-type fixed rest)
+  (declare (type approximate-fun-type call-type)
+           (list fixed)
+           (type (or ctype null) rest))
   (do ((types (approximate-fun-type-types call-type) (cdr types))
        (n 1 (1+ n))
        (arg fixed (cdr arg)))
@@ -556,9 +567,8 @@ and no value was provided for it." name))))))))))
 
 ;;; Check that each of the call-types is compatible with DECL-TYPE,
 ;;; complaining if not or if we can't tell.
-(declaim (ftype (function (list ctype string &rest t) (values))
-                check-approximate-arg-type))
 (defun check-approximate-arg-type (call-types decl-type context &rest args)
+  (declare (list call-types) (type ctype decl-type) (string context))
   (when *ctype-test-fun*
     (let ((losers *empty-type*))
       (dolist (ctype call-types)
@@ -647,20 +657,10 @@ and no value was provided for it." name))))))))))
 ;;; defaults. Returning the actual vars allows us to use the right
 ;;; variable name in warnings.
 ;;;
-;;; A slightly subtle point: with keywords and optionals, the type in
-;;; the function type is only an assertion on calls --- it doesn't
-;;; constrain the type of default values. So we have to union in the
-;;; type of the default. With optionals, we can't do any assertion
-;;; unless the default is constant.
-;;;
-;;; With keywords, we exploit our knowledge about how hairy keyword
-;;; defaulting is done when computing the type assertion to put on the
-;;; main-entry argument. In the case of hairy keywords, the default
-;;; has been clobbered with NIL, which is the value of the main-entry
-;;; arg in the unsupplied case, whatever the actual default value is.
-;;; So we can just assume the default is constant, effectively
-;;; unioning in NULL, and not totally blow off doing any type
-;;; assertion.
+;; Despite FTYPE proclamations affecting only the calls and not the
+;; function itself, it would be very weird to see a default of &key or
+;; &optional be different from the proclaimed type. Accept NULL only
+;; when there's no default form.
 (defun find-optional-dispatch-types (od type where)
   (declare (type optional-dispatch od)
            (type fun-type type)
@@ -711,47 +711,50 @@ and no value was provided for it." name))))))))))
             (arglist (optional-dispatch-arglist od)))
         (dolist (arg arglist)
           (cond
-           ((lambda-var-arg-info arg)
-            (let* ((info (lambda-var-arg-info arg))
-                   (default (arg-info-default info))
-                   (def-type (when (constantp default)
-                               (ctype-of (constant-form-value default)))))
-              (ecase (arg-info-kind info)
-                (:keyword
-                 (let* ((key (arg-info-key info))
-                        (kinfo (find key keys :key #'key-info-name)))
-                   (cond
-                    (kinfo
-                     (res (type-union (key-info-type kinfo)
-                                      (or def-type (specifier-type 'null)))))
-                    (t
-                     (note-lossage
-                      "Defining a ~S keyword not present in ~A."
-                      key where)
-                     (res *universal-type*)))))
-                (:required (res (pop type-required)))
-                (:optional
-                 ;; We can exhaust TYPE-OPTIONAL when the type was
-                 ;; simplified as described above.
-                 (res (type-union (or (pop type-optional)
-                                      *universal-type*)
-                                  (or def-type *universal-type*))))
-                (:rest
-                 (when (fun-type-rest type)
-                   (res (specifier-type 'list))))
-                (:more-context
-                 (when (fun-type-rest type)
-                   (res *universal-type*)))
-                (:more-count
-                 (when (fun-type-rest type)
-                   (res (specifier-type 'fixnum)))))
-              (vars arg)
-              (when (arg-info-supplied-p info)
-                (res *universal-type*)
-                (vars (arg-info-supplied-p info)))))
-           (t
-            (res (pop type-required))
-            (vars arg))))
+            ((lambda-var-arg-info arg)
+             (let* ((info (lambda-var-arg-info arg))
+                    (default-p (arg-info-default-p info)))
+               (ecase (arg-info-kind info)
+                 (:keyword
+                  (let* ((key (arg-info-key info))
+                         (kinfo (find key keys :key #'key-info-name)))
+                    (cond
+                      (kinfo
+                       (res (if default-p
+                                (key-info-type kinfo)
+                                (type-union (key-info-type kinfo)
+                                            (specifier-type 'null)))))
+                      (t
+                       (note-lossage
+                        "Defining a ~S keyword not present in ~A."
+                        key where)
+                       (res *universal-type*)))))
+                 (:required (res (pop type-required)))
+                 (:optional
+                  ;; We can exhaust TYPE-OPTIONAL when the type was
+                  ;; simplified as described above.
+                  (res (let ((type (or (pop type-optional)
+                                       *universal-type*)))
+                         (if default-p
+                             type
+                             (type-union type
+                                         (specifier-type 'null))))))
+                 (:rest
+                  (when (fun-type-rest type)
+                    (res (specifier-type 'list))))
+                 (:more-context
+                  (when (fun-type-rest type)
+                    (res *universal-type*)))
+                 (:more-count
+                  (when (fun-type-rest type)
+                    (res (specifier-type 'fixnum)))))
+               (vars arg)
+               (when (arg-info-supplied-p info)
+                 (res *universal-type*)
+                 (vars (arg-info-supplied-p info)))))
+            (t
+             (res (pop type-required))
+             (vars arg))))
 
         (dolist (key keys)
           (unless (find (key-info-name key) arglist
@@ -833,7 +836,7 @@ and no value was provided for it." name))))))))))
                             (not (member :result (cdr really-assert)))))
                (assert-lvar-type (return-result return) type-returns
                                  policy
-                                 :ftype)))
+                                 'ftype-context)))
            (loop for var in vars
                  for type in types do
                  (cond ((basic-var-sets var)
@@ -876,26 +879,6 @@ and no value was provided for it." name))))))))))
     ((t) nil)
     (t `(:not . ,explicit-check))))
 
-;;; FIXME: This is quite similar to ASSERT-NEW-DEFINITION.
-(defun assert-global-function-definition-type (name fun)
-  (declare (type functional fun))
-  (let ((where (info :function :where-from name))
-        (explicit-check (getf (functional-plist fun) 'explicit-check)))
-    (if (eq where :declared)
-        (let ((type
-               (massage-global-definition-type (global-ftype name) fun)))
-          (setf (leaf-type fun) type)
-          (assert-definition-type
-           fun type
-           :unwinnage-fun #'compiler-notify
-           :where "proclamation"
-           :really-assert (explicit-check->really-assert explicit-check)))
-        ;; Can't actually test this. DEFSTRUCTs declare this, but non-toplevel
-        ;; ones won't have an FTYPE at compile-time.
-        #+nil
-        (when explicit-check
-          (warn "Explicit-check without known FTYPE is meaningless")))))
-
 ;;; If the function has both &REST and &KEY, FIND-OPTIONAL-DISPATCH-TYPES
 ;;; doesn't complain about the type missing &REST -- which is good, because in
 ;;; that case &REST is really an implementation detail and not part of the
@@ -1078,7 +1061,7 @@ and no value was provided for it." name))))))))))
 
 (defun %compile-time-type-error (values atype dtype detail code-context cast-context)
   (declare (ignore dtype))
-  (cond ((eq cast-context :ftype)
+  (cond ((eq cast-context 'ftype-context)
          (error 'simple-type-error
                 :datum (car values)
                 :expected-type atype
@@ -1140,10 +1123,11 @@ and no value was provided for it." name))))))))))
                                            (condition 'type-warning))
   (let ((*compiler-error-context* node))
     (cond ((eq atype nil))
-          ((eq cast-context :ftype)
+          ((eq cast-context 'ftype-context)
            (warn condition
                  :format-control
-                 "~@<Derived type of ~S is ~2I~_~S, ~
+                 "~@<Derived type of ~/sb-impl:print-type-specifier/ is ~
+                    ~2I~_~/sb-impl:print-type-specifier/, ~
                     ~I~_conflicting with the declared function return type ~
                     ~2I~_~/sb-impl:print-type-specifier/.~@:>"
                  :format-arguments (list detail dtype atype)))
@@ -1158,7 +1142,8 @@ and no value was provided for it." name))))))))))
                        :format-arguments (list (constant-form-value detail) atype))
                  (warn condition
                        :format-control
-                       "~@<Derived type of ~S is ~2I~_~S, ~
+                       "~@<Derived type of ~/sb-impl:print-type-specifier/ is ~
+                            ~2I~_~/sb-impl:print-type-specifier/, ~
                             ~I~_conflicting with its asserted type ~
                             ~2I~_~/sb-impl:print-type-specifier/.~@:>"
                        :format-arguments (list detail dtype atype)))))
@@ -1172,30 +1157,3 @@ and no value was provided for it." name))))))))))
                       ~2I~_~/sb-impl:print-type-specifier/.~@:>"
                  :format-arguments (list (rest detail) (first detail)
                                          dtype atype))))))
-
-
-(defoptimizer (%compile-time-type-error ir2-convert)
-    ((objects atype dtype detail code-context cast-context) node block)
-  (declare (ignore objects code-context))
-  ;; Remove %COMPILE-TIME-TYPE-ERROR bits
-  (setf (node-source-path node)
-        (cdr (node-source-path node)))
-  (%compile-time-type-error-warn node
-                                 (lvar-value atype)
-                                 (lvar-value dtype)
-                                 (lvar-value detail)
-                                 :cast-context (lvar-value cast-context))
-  (ir2-convert-full-call node block))
-
-(defoptimizer (%compile-time-type-style-warn ir2-convert)
-    ((objects atype dtype detail code-context cast-context) node block)
-  (declare (ignore objects code-context block))
-  ;; Remove %COMPILE-TIME-TYPE-ERROR bits
-  (setf (node-source-path node)
-        (cddr (node-source-path node)))
-  (%compile-time-type-error-warn node
-                                 (lvar-value atype)
-                                 (lvar-value dtype)
-                                 (lvar-value detail)
-                                 :cast-context (lvar-value cast-context)
-                                 :condition 'type-style-warning))
