@@ -38,9 +38,7 @@
   ;; used for everything else. We use a separate EQ table to avoid
   ;; performance pathologies with objects for which SIMILAR
   ;; degenerates to EQL. Everything entered in the SIMILAR table is
-  ;; also entered in the EQ table. The NAMED-CONSTANT is used for
-  ;; named constants whose references are dumped as load time values
-  ;; of SYMBOL-GLOBAL-VALUE.
+  ;; also entered in the EQ table.
   (similar-table (make-similarity-table) :type hash-table :read-only t)
   (eq-table (make-hash-table :test 'eq) :type hash-table :read-only t)
   ;; the INSTANCE table maps dumpable instances to unique IDs for calculating
@@ -428,9 +426,7 @@
     ;; sanity checks
     (aver (zerop (hash-table-count (fasl-output-patch-table fasl-output))))
     ;; End the group.
-    (dump-fop 'fop-verify-empty-stack fasl-output)
-    (dump-fop 'fop-verify-table-size fasl-output (fasl-output-table-free fasl-output))
-    (dump-fop 'fop-end-group fasl-output))
+    (dump-fop 'fop-end-group fasl-output (fasl-output-table-free fasl-output)))
 
   ;; That's all, folks.
   (close (fasl-output-stream fasl-output) :abort abort-p)
@@ -726,7 +722,7 @@
   (declare (inline assoc))
   (cond ((cdr (assoc pkg (fasl-output-packages file) :test #'eq)))
         (t
-         (let ((s (package-name pkg)))
+         (let ((s (sb-xc:package-name pkg)))
            (dump-fop 'fop-named-package-save file (length s))
            ;; Package names are always dumped as varint-encoded character strings
            ;; except on non-unicode builds.
@@ -1033,48 +1029,72 @@
   (assert (<= (length +fixup-kinds+) 8))) ; fixup-kind fits in 3 bits
 
 (defconstant-eqx +fixup-flavors+
-  #(:assembly-routine :assembly-routine* :asm-routine-nil-offset
+  #(:assembly-routine :assembly-routine*
     :gc-barrier :symbol-tls-index
-    :foreign :foreign-dataref :code-object
-    :layout :immobile-symbol :named-call :static-call
+    :alien-code-linkage-index :alien-data-linkage-index
+    :foreign :foreign-dataref
+    :code-object
+    :layout :immobile-symbol :fdefn-call :static-call
     :symbol-value
     :layout-id)
   #'equalp)
 
 ;;; Pack the aspects of a fixup into an integer.
+;;; DATA is for asm routine fixups. The routine can be represented in 8 bits,
+;;; so the fixup can be reduced to one word instead of an integer and a symbol.
 (declaim (inline !pack-fixup-info))
-(defun !pack-fixup-info (offset kind flavor)
-  ;; ARM gets "error during constant folding"
-  #+arm (declare (notinline position))
-  (logior (ash (the (mod 16) (or (position flavor +fixup-flavors+)
-                                 (error "Bad fixup flavor ~s" flavor)))
-               3)
+(defun !pack-fixup-info (offset kind flavor data)
+  (logior ;; 3 bits
           (the (mod 8) (or (position kind +fixup-kinds+)
                            (error "Bad fixup kind ~s" kind)))
-          (ash offset 7)))
+          ;; 4 bits
+          (ash (the (mod 16) (or (position flavor +fixup-flavors+)
+                                 (error "Bad fixup flavor ~s" flavor)))
+               3)
+          ;; 8 bits
+          (ash (the (mod 256) data) 7)
+          ;; whatever it needs
+          (ash offset 15)))
 
 ;;; Unpack an integer from DUMP-FIXUPs. Shared by genesis and target fasloader
 (declaim (inline !unpack-fixup-info))
-(defun !unpack-fixup-info (packed-info) ; Return (VALUES offset kind flavor)
-  ;; ARM gets "error during constant folding"
-  #+arm (declare (notinline aref))
-  (values (ash packed-info -7)
+(defun !unpack-fixup-info (packed-info) ; Return (VALUES offset kind flavor data)
+  (values (ash packed-info -15)
           (aref +fixup-kinds+ (ldb (byte 3 0) packed-info))
-          (aref +fixup-flavors+ (ldb (byte 4 3) packed-info))))
+          (aref +fixup-flavors+ (ldb (byte 4 3) packed-info))
+          (ldb (byte 8 7) packed-info)))
+
+#+(or arm arm64 riscv sparc) ; these don't have any retained packed fixups (yet)
+(defun sb-c::pack-retained-fixups (fixup-notes)
+  (declare (ignore fixup-notes))
+  0) ; as if from PACK-CODE-FIXUP-LOCS
 
 ;;; Dump all the fixups.
 ;;;  - foreign (C) symbols: named by a string
 ;;;  - code object references: don't need a name.
 ;;;  - everything else: a symbol for the name.
-(defun dump-fixups (fixups fasl-output &aux (n 0))
-  (declare (list fixups) (type fasl-output fasl-output))
-  (dolist (note fixups n)
+(defun dump-fixups (fixup-notes alloc-points fasl-output &aux (nelements 2))
+  (declare (type list fixup-notes) (type fasl-output fasl-output))
+  ;; "retained" fixups are those whose offset in the code needs to be
+  ;; remembered for subsequent reapplication by the garbage collector,
+  ;; or in some cases, on core startup.
+  (dump-object (sb-c::pack-retained-fixups fixup-notes) fasl-output)
+  (dump-object alloc-points fasl-output)
+  (dolist (note fixup-notes nelements)
     (let* ((fixup (fixup-note-fixup note))
            (name (fixup-name fixup))
            (flavor (fixup-flavor fixup))
-           (info (!pack-fixup-info (fixup-note-position note)
-                                   (fixup-note-kind note)
-                                   flavor))
+           (named (not (member flavor '(:code-object :gc-barrier))))
+           (data
+            (or #-sb-xc-host ; ASM routine indices aren't known to the cross-compiler
+                (when (member flavor '(:assembly-routine :assembly-routine*))
+                  (setq named nil)
+                  (the (integer 1 *) ; data can't be 0
+                       (cddr (gethash name (%asm-routine-table *assembler-routines*)))))
+                0))
+           (info
+            (!pack-fixup-info (fixup-note-position note) (fixup-note-kind note)
+                              flavor data))
            (operand
             (ecase flavor
               ((:code-object :gc-barrier) (the null name))
@@ -1087,7 +1107,7 @@
                           (t name)))))
               (:layout-id
                (the wrapper name))
-              ((:assembly-routine :assembly-routine* :asm-routine-nil-offset
+              ((:assembly-routine :assembly-routine*
                :symbol-tls-index
                ;; Only #+immobile-space can use the following two flavors.
                ;; An :IMMOBILE-SYMBOL fixup references the symbol itself,
@@ -1096,11 +1116,12 @@
                ;; but its global value must be an immobile object.
                :immobile-symbol :symbol-value)
                (the symbol name))
-              ((:foreign :foreign-dataref) (the string name))
-              ((:named-call :static-call) name))))
-      (dump-object operand fasl-output)
-      (dump-integer info fasl-output))
-    (incf n)))
+              ((:alien-code-linkage-index :alien-data-linkage-index
+                :foreign :foreign-dataref) (the string name))
+              ((:fdefn-call :static-call) name))))
+      (dump-object info fasl-output)
+      (incf nelements (cond (named (dump-object operand fasl-output) 2)
+                            (t 1))))))
 
 ;;; Dump out the constant pool and code-vector for component, push the
 ;;; result in the table, and return the offset.
@@ -1118,12 +1139,10 @@
   (declare (type component component)
            (type index code-length)
            (type fasl-output fasl-output))
-  (let* ((n-fixups (dump-fixups fixups fasl-output))
-         (2comp (component-info component))
+  (let* ((2comp (component-info component))
          (constants (sb-c:ir2-component-constants 2comp))
          (header-length (length constants))
-         (n-named-calls 0))
-    (dump-object alloc-points fasl-output)
+         (n-fdefns 0))
     (collect ((patches)
               (named-constants))
       ;; Dump the constants, noting any :ENTRY constants that have to
@@ -1155,8 +1174,12 @@
                     (dump-fop 'fop-misc-trap fasl-output)))))
                (:load-time-value
                 (dump-push (cadr entry) fasl-output))
-               ((:named-call :fdefinition)
-                (when (eq (car entry) :named-call) (incf n-named-calls))
+               (:fdefinition
+                ;; It's possible for other fdefns to be found in the header, but they can't
+                ;; have resulted from IR2 conversion. They would have had to come from
+                ;; something like (load-time-value (find-or-create-fdefn ...))
+                ;; which is fine, but they don't count for this purpose.
+                (incf n-fdefns)
                 (dump-object (cadr entry) fasl-output)
                 (dump-fop 'fop-fdefn fasl-output))
                (:known-fun
@@ -1167,7 +1190,11 @@
                 (dump-specialized-vector (make-array (cdr entry)
                                                      :element-type '(unsigned-byte 8)
                                                      :initial-element #xFF)
-                                         fasl-output))))
+                                         fasl-output))
+               #+arm64
+               (:tls-index
+                (dump-object (cdr entry) fasl-output)
+                (dump-fop 'fop-tls-index fasl-output))))
             (null
              (dump-fop 'fop-misc-trap fasl-output)))))
 
@@ -1178,16 +1205,18 @@
         (push (dump-to-table fasl-output)
               (fasl-output-debug-info fasl-output)))
 
-      (dump-fop 'fop-load-code fasl-output
-                (logior (ash header-length 1)
-                        (if (sb-c::code-immobile-p component) 1 0))
-                code-length n-fixups)
+      (let ((n-fixup-elts (dump-fixups fixups alloc-points fasl-output)))
+        (dump-fop 'fop-load-code fasl-output
+                  (logior (ash header-length 1)
+                          (if (sb-c::code-immobile-p component) 1 0))
+                  code-length
+                  n-fixup-elts))
       ;; Fasl dumper/loader convention allows at most 3 integer args.
       ;; Others have to be written with explicit calls.
       (dump-integer-as-n-bytes (length (sb-c::ir2-component-entries 2comp))
                                4 ; output 4 bytes
                                fasl-output)
-      (dump-integer-as-n-bytes (the (unsigned-byte 22) n-named-calls)
+      (dump-integer-as-n-bytes (the (unsigned-byte 22) n-fdefns)
                                4 ; output 4 bytes
                                fasl-output)
       (dump-segment code-segment code-length fasl-output)
@@ -1206,8 +1235,7 @@
 ;;; This is only called from assemfile, which doesn't exist in the target.
 #+sb-xc-host
 (defun dump-assembler-routines (code-segment octets fixups alloc-points routines file)
-  (let ((n-fixups (dump-fixups fixups file)))
-    (dump-object alloc-points file)
+  (let ((n-fixup-elts (dump-fixups fixups alloc-points file)))
     ;; The name -> address table has to be created before applying fixups
     ;; because a fixup may refer to an entry point in the same code component.
     ;; So these go on the stack last, i.e. nearest the top.
@@ -1215,12 +1243,12 @@
     ;; except possibly when there are multiple entry points to one routine
     (dolist (routine (reverse routines))
       (dump-object (car routine) file)
-      (dump-integer (+ (label-position (cadr routine))
-                       (caddr routine))
+      (dump-integer (+ (label-position (cadr routine)) (caddr routine))
                     file))
     (dump-fop 'fop-assembler-code file)
-    (dolist (word (list (length octets) (length routines) n-fixups))
-      (dump-word word file))
+    (dump-word (length routines) file)
+    (dump-word (length octets) file)
+    (dump-word n-fixup-elts file)
     (write-segment-contents code-segment (fasl-output-stream file))
     (dump-pop file)))
 
@@ -1240,8 +1268,6 @@
   (declare (type component component))
   (declare (type fasl-output file))
 
-  (dump-fop 'fop-verify-table-size file (fasl-output-table-free file))
-
   #+sb-dyncount
   (let ((info (sb-c::ir2-component-dyncount-info (component-info component))))
     (when info
