@@ -16,14 +16,22 @@
 (in-package "SB-C")
 
 ;;; ANSI limits on compilation
-(defconstant call-arguments-limit most-positive-fixnum
+;;; On AMD64 we prefer to use the ECX (not RCX) register,
+;;; which means that there can only be 32 bits of precision,
+;;; so accounting for the fixnum tag and 1 bit for the sign,
+;;; this leaves 30 bits. Of course this number is ridiculous
+;;; as a call with that many args would consume 8 GB of stack,
+;;; but it's surely not as ridiculous as MOST-POSITIVE-FIXNUM.
+(defconstant call-arguments-limit
+  #+x86-64 (ash 1 30)
+  #-x86-64 most-positive-fixnum
   "The exclusive upper bound on the number of arguments which may be passed
   to a function, including &REST args.")
-(defconstant lambda-parameters-limit most-positive-fixnum
+(defconstant lambda-parameters-limit call-arguments-limit
   "The exclusive upper bound on the number of parameters which may be specified
   in a given lambda list. This is actually the limit on required and &OPTIONAL
   parameters. With &KEY and &AUX you can get more.")
-(defconstant multiple-values-limit most-positive-fixnum
+(defconstant multiple-values-limit call-arguments-limit
   "The exclusive upper bound on the number of multiple VALUES that you can
   return.")
 
@@ -41,32 +49,28 @@
   (expansion nil :read-only t))
 (declaim (freeze-type dxable-args))
 
-;;; FREE-VARS translates from the names of variables referenced
-;;; globally to the LEAF structures for them. FREE-FUNS is like
-;;; FREE-VARS, only it deals with function names.
-;;;
-;;; We must preserve the property that a proclamation for a global
-;;; thing only affects the code after it. This takes some work, since
-;;; a proclamation may appear in the middle of a block being
-;;; compiled. If there are references before the proclaim, then we
-;;; copy the current entry before modifying it. Code converted before
-;;; the proclaim sees the old Leaf, while code after it sees the new
-;;; LEAF.
 (defstruct (ir1-namespace (:conc-name "") (:copier nil) (:predicate nil))
+  ;; FREE-VARS translates from the names of variables referenced
+  ;; globally to the LEAF structures for them. FREE-FUNS is like
+  ;; FREE-VARS, only it deals with function names.
+  ;;
+  ;; We must preserve the property that a proclamation for a global
+  ;; thing only affects the code after it. This takes some work, since
+  ;; a proclamation may appear in the middle of a block being
+  ;; compiled. If there are references before the proclaim, then we
+  ;; copy the current entry before modifying it. Code converted before
+  ;; the proclaim sees the old Leaf, while code after it sees the new
+  ;; LEAF.
   (free-vars (make-hash-table :test 'eq) :read-only t :type hash-table)
   (free-funs (make-hash-table :test #'equal) :read-only t :type hash-table)
-  ;; These hashtables translate from constants to the LEAFs that
-  ;; represent them.
-  ;; Table 1: one entry per named constant
-  (named-constants (make-hash-table :test 'eq) :read-only t :type hash-table)
-  ;; Table 2: one entry for each unnamed constant as compared by EQL
+  ;; We use the same CONSTANT structure to represent all EQL anonymous
+  ;; constants. This hashtable translates from constants to the LEAFs
+  ;; that represent them.
   (eql-constants (make-hash-table :test 'eql) :read-only t :type hash-table)
-  ;; Table 3: one key per EQUAL constant,
-  ;; with the caveat that lookups must discriminate amongst constants that
-  ;; are EQUAL but not similar.  The value in the hash-table is a list of candidates
-  ;; (#<constant1> #<constant2> ... #<constantN>) such that CONSTANT-VALUE
-  ;; of each is EQUAL to the key for the hash-table entry, but dissimilar
-  ;; from each other. Notably, strings of different element types can't be similar.
+  ;; During file compilation we are allowed to coalesce similar
+  ;; constants. This coalescing is distinct from the coalescing done
+  ;; in the dumper, since the effect here is to reduce the number of
+  ;; boxed constants appearing in a code component.
   (similar-constants (sb-fasl::make-similarity-table) :read-only t :type hash-table))
 (declaim (freeze-type ir1-namespace))
 
@@ -85,7 +89,7 @@
 ;;; Bind this to a stream to capture various internal debugging output.
 (defvar *compiler-trace-output* nil)
 ;;; These are the default, but the list can also include
-;;; :pre-ir2-optimize, :symbolic-asm, and :sb-graph.
+;;; :pre-ir2-optimize, :symbolic-asm.
 (defvar *compile-trace-targets* '(:ir1 :ir2 :vop :symbolic-asm :disassemble))
 (defvar *current-path*)
 (defvar *current-component*)
@@ -160,6 +164,15 @@
 (declaim (always-bound *compile-time-eval*))
 
 #-immobile-code (defmacro code-immobile-p (thing) `(progn ,thing nil))
+#-sb-xc-host ; not needed for make-hlst-1
+(defmacro maybe-with-system-tlab ((source-object) allocator)
+  (declare (ignorable source-object))
+  #+system-tlabs `(if (sb-vm::force-to-heap-p ,source-object)
+                      (locally (declare (sb-c::tlab :system)) ,allocator)
+                      ,allocator)
+  #-system-tlabs allocator)
+;;; TLAB selection is an aspect of a POLICY but this sets the global choice
+(defvar *force-system-tlab* nil)
 
 ;;; The allocation quantum for boxed code header words.
 ;;; 2 implies an even length boxed header; 1 implies no restriction.
@@ -223,7 +236,11 @@
   ;; We build a list of top-level lambdas, and then periodically smash them
   ;; together into a single component and compile it.
   (pending-toplevel-lambdas nil :type list)
-
+  ;; We record whether the package environment has changed during the
+  ;; compilation of some sequence top level forms. This allows the
+  ;; compiler to dump symbols in such a way that the loader can
+  ;; reconstruct them in the correct package.
+  (package-environment-changed nil :type boolean)
   ;; Bidrectional map between IR1/IR2/assembler abstractions and a corresponding
   ;; small integer or string identifier. One direction could be done by adding
   ;; the ID as slot to each object, but we want both directions.
