@@ -76,6 +76,7 @@
     (and static-fun-index
          (+ (* (length +static-symbols+) (pad-data-block symbol-size))
             (pad-data-block (1- symbol-size))
+            (* 4 n-word-bytes) ; sizeof SB-LOCKLESS:+TAIL+
             (- list-pointer-lowtag)
             (* static-fun-index (pad-data-block fdefn-size))
             other-pointer-lowtag))))
@@ -180,10 +181,8 @@
 
 ;;; Make a TN to hold the number-stack frame pointer.  This is allocated
 ;;; once per component, and is component-live.
-(defun make-nfp-tn ()
-  #+c-stack-is-control-stack
-  (make-restricted-tn *fixnum-primitive-type* ignore-me-sc-number)
   #-c-stack-is-control-stack
+(defun make-nfp-tn ()
   (component-live-tn
    (make-wired-tn *fixnum-primitive-type* immediate-arg-scn nfp-offset)))
 
@@ -200,7 +199,7 @@
 
 (defun make-number-stack-pointer-tn ()
   #+c-stack-is-control-stack
-  (make-restricted-tn *fixnum-primitive-type* ignore-me-sc-number)
+  (make-restricted-tn *fixnum-primitive-type* any-reg-sc-number)
   #-c-stack-is-control-stack
   (make-normal-tn *fixnum-primitive-type*))
 
@@ -308,8 +307,9 @@
         (return-from stack-consed-p nil))
       (when (eq (vop-name vop) 'allocate-vector-on-stack)
         (return-from stack-consed-p t))
-      (when (and (eq (vop-name vop) 'fixed-alloc)
-                 (fifth (vop-codegen-info vop))) ; STACK-ALLOCATE-P
+      (when (or (and (eq (vop-name vop) 'fixed-alloc) ; do we still need this case?
+                     (fifth (vop-codegen-info vop))) ; STACK-ALLOCATE-P
+                (eq (vop-name vop) 'sb-c::fixed-alloc-to-stack))
         (return-from stack-consed-p t))
       ;; Should we try to detect a stack-consed LIST also?
       ;; I don't think that will work.
@@ -392,9 +392,7 @@
   (logand (1- rank) array-rank-mask))
 
 (defun compute-object-header (nwords widetag-or-metadata)
-  (let* ((widetag (if (typep widetag-or-metadata '(or wrapper defstruct-description))
-                      instance-widetag
-                      widetag-or-metadata))
+  (let* ((widetag (if (fixnump widetag-or-metadata) widetag-or-metadata instance-widetag))
          (array-header-p
           (or (= widetag simple-array-widetag)
               (>= widetag complex-base-string-widetag))))
@@ -413,3 +411,54 @@
 (defmacro id-bits-offset ()
   (let ((slot (get-dsd-index layout sb-kernel::id-word0)))
     (ash (+ sb-vm:instance-slots-offset slot) sb-vm:word-shift)))
+
+;;; I'd like the division-by-constant-integer optimization to work
+;;; during cross-compilation, but the algorithm to compute the magic
+;;; parameters is expressed in C, not Lisp. I need to translate it.
+#-sb-xc-host
+(defun sb-c:compute-udiv32-magic (divisor)
+  (with-alien ((mag (struct magu
+                            (m unsigned-int)
+                            (a int)
+                            (s int)))
+               (compute-udiv-magic32 (function void int (* (struct magu)))
+                                     :extern))
+    (alien-funcall compute-udiv-magic32 divisor (addr mag))
+    (values (slot mag 'm) (slot mag 'a) (slot mag 's))))
+
+;;; "Algorithm 2: Algorithm to select the number of fractional bits and the scaled
+;;; approximate reciprocal in the case of unsigned integers."
+;;; from https://r-libre.teluq.ca/1633/1/Faster_Remainder_of_the_Division_by_a_Constant.pdf
+;;; See also https://github.com/bmkessler/fastdiv for that coded in Go.
+;;; D = divisor
+;;; N = number of bits of precision in numerator
+;;; FRACTION-BITS is what you want, or :VARIABLE for the smallest
+;;;
+;;; Note that for 32 fraction bits, the divisor can *not* use all 32 bits of precision.
+;;; It can only have about 27 or 28 significant bits. This function will figure it out.
+(defun compute-fastrem-coefficient (d n fraction-bits)
+  (multiple-value-bind (smallest-f c)
+      (flet ((is-pow2 (n)
+               (declare (unsigned-byte n))
+               (let ((l (integer-length n)))
+                 (= n (ash 1 (1- l))))))
+        (if (is-pow2 d)
+            (values (1- (integer-length d)) 1)
+            (loop for L from 0
+                  do (let* ((F (+ N L))
+                            (2^F (expt 2 F)))
+                       (when (<= d (+ (mod 2^F d) (expt 2 L)))
+                         (let ((c (ceiling (expt 2 F) d)))
+                           (return (values F c))))))))
+    (cond ((eq fraction-bits :variable) ; return the smallest F
+           (values c smallest-f))
+          (t
+           ;; Otherwise hardwire F to 32 so the algorithm can use :DWORD
+           ;; register moves to perform the shifting and masking.
+           ;; But make sure the smallest-f is not more than 32, or else
+           ;; this can't work.
+           (when (> smallest-f fraction-bits)
+             (error "Need ~D fraction bits for divisor ~D and ~D bit dividend"
+                    smallest-f d n))
+           (values (ceiling (expt 2 fraction-bits) d) fraction-bits)))))
+
