@@ -56,14 +56,11 @@
            #+long-float simple-array-complex-long-float-p
            simple-rank-1-array-*-p
            system-area-pointer-p realp
-           ;; #-64-bit
+           signed-byte-8-p signed-byte-16-p
            unsigned-byte-32-p
-           ;; #-64-bit
            signed-byte-32-p
-           #+64-bit
-           unsigned-byte-64-p
-           #+64-bit
-           signed-byte-64-p
+           #+64-bit unsigned-byte-64-p
+           #+64-bit signed-byte-64-p
            weak-pointer-p code-component-p lra-p
            sb-int:unbound-marker-p
            pointerp
@@ -74,6 +71,8 @@
            non-null-symbol-p)
     (t) boolean (movable foldable flushable))
 
+(defknown car-eq-if-listp (t t) boolean (movable foldable flushable))
+
 (defknown #.(loop for (name) in *vector-without-complex-typecode-infos*
                   collect name)
     (t) boolean (movable foldable flushable))
@@ -95,14 +94,24 @@
 (defknown pointer-hash (t) fixnum (flushable))
 
 (defknown %sp-string-compare
-  (simple-string index (or null index) simple-string index (or null index))
+  (simple-string simple-string index (or null index) index (or null index))
   (values index fixnum)
   (foldable flushable no-verify-arg-count))
 (defknown %sp-string=
-  (simple-string index (or null index) simple-string index (or null index))
+  (simple-string simple-string index (or null index) index (or null index))
   boolean
   (foldable flushable no-verify-arg-count))
 
+(defknown (sb-impl::instance-sxhash sb-impl::%instance-sxhash)
+    (instance) hash-code (flushable))
+;;; SXHASH values on numbers and strings are predictable, therefore the next batch
+;;; of functions are flushable. Perhaps not entirely obviously, symbol hashes are
+;;; predictable because we hash by name. And while ENSURE-SYMBOL-HASH is not
+;;; - strictly speaking - a side-effectless operation, it has no effect as far as
+;;; user-written code is concerned. i.e. nothing portably expressed can discern
+;;; whether memoization of the hash occurred. (I feel like we would have a cleaner
+;;; implementation if we just eagerly compute the hash anyway. Practically all
+;;; symbols eventually get hashed)
 (defknown sb-impl::number-sxhash (number) hash-code (foldable flushable))
 (defknown %sxhash-string (string) hash-code (foldable flushable))
 (defknown %sxhash-simple-string (simple-string) hash-code (foldable flushable))
@@ -129,7 +138,7 @@
 
 ;;; SYMBOL-PACKAGE-ID for #+compact-symbol demands a vop which avoids loading
 ;;; a raw bit value in a descriptor register (the SLOT vop returns a descriptor)
-(defknown sb-impl::symbol-package-id (symbol) (unsigned-byte 16))
+(defknown symbol-package-id (symbol) (unsigned-byte 16))
 ;;; TODO: I'd like to eliminate the (OR NULL) from this return type.
 ;;; For that to happen, I probably need +nil-packed-infos+ to become
 ;;; placed in static space because assembly routines may need it.
@@ -242,7 +251,7 @@
 #+(or arm64 ppc ppc64 riscv x86 x86-64)
 (defknown %raw-instance-cas/word (instance index sb-vm:word sb-vm:word)
   sb-vm:word ())
-#+riscv
+#+(or arm64 riscv x86 x86-64)
 (defknown %raw-instance-cas/signed-word (instance index sb-vm:signed-word sb-vm:signed-word)
   sb-vm:signed-word ())
 (defknown %raw-instance-xchg/word (instance index sb-vm:word) sb-vm:word ())
@@ -547,6 +556,7 @@
 
 (defknown %bignum-ref (bignum bignum-index) bignum-element-type
   (flushable #-bignum-assertions always-translatable))
+
 #+(or x86 x86-64)
 (defknown %bignum-ref-with-offset (bignum fixnum (signed-byte 24))
   bignum-element-type (flushable always-translatable))
@@ -555,18 +565,25 @@
   (values)
   (#-bignum-assertions always-translatable))
 
+(defknown %half-bignum-ref (bignum sb-kernel::half-bignum-index) sb-kernel::half-bignum-element-type
+    (flushable #-bignum-assertions always-translatable))
+
+(defknown (setf %half-bignum-ref) (sb-kernel::half-bignum-element-type bignum sb-kernel::half-bignum-index)
+    (values)
+    (#-bignum-assertions always-translatable))
+
 ;;; Return T if digit is positive, or NIL if negative.
 (defknown %digit-0-or-plusp (bignum-element-type) boolean
-  (foldable flushable movable always-translatable))
+    (foldable flushable movable always-translatable))
 
 ;;; %ADD-WITH-CARRY returns a bignum digit and a carry resulting from adding
 ;;; together a, b, and an incoming carry.
 ;;; %SUBTRACT-WITH-BOROW returns a bignum digit and a borrow resulting from
 ;;; subtracting b from a, and subtracting a possible incoming borrow.
 (defknown (%add-with-carry %subtract-with-borrow)
-          (bignum-element-type bignum-element-type (mod 2))
-  (values bignum-element-type (mod 2))
-  (foldable flushable movable always-translatable))
+    (bignum-element-type bignum-element-type (mod 2))
+    (values bignum-element-type (mod 2))
+    (foldable flushable movable always-translatable))
 
 ;;; This multiplies x-digit and y-digit, producing high and low digits
 ;;; manifesting the result. Then it adds the low digit, res-digit, and
@@ -579,19 +596,19 @@
 ;;; accumulating partial results which is where the res-digit comes
 ;;; from.
 (defknown %multiply-and-add
-          (bignum-element-type bignum-element-type bignum-element-type
-                               &optional bignum-element-type)
-  (values bignum-element-type bignum-element-type)
-  (foldable flushable movable always-translatable))
+    (bignum-element-type bignum-element-type bignum-element-type
+                         &optional bignum-element-type)
+    (values bignum-element-type bignum-element-type)
+    (foldable flushable movable always-translatable))
 
 ;;; Multiply two digit-size numbers, returning a 2*digit-size result
 ;;; split into two digit-size quantities.
 (defknown %multiply (bignum-element-type bignum-element-type)
-  (values bignum-element-type bignum-element-type)
-  (foldable flushable movable always-translatable))
+    (values bignum-element-type bignum-element-type)
+    (foldable flushable movable always-translatable))
 
 (defknown %lognot (bignum-element-type) bignum-element-type
-  (foldable flushable movable always-translatable))
+    (foldable flushable movable always-translatable))
 
 (defknown (%logand %logior %logxor) (bignum-element-type bignum-element-type)
   bignum-element-type
@@ -606,11 +623,15 @@
   (values bignum-element-type bignum-element-type)
   (foldable flushable movable always-translatable))
 
+(defknown %half-bigfloor (half-bignum-element-type half-bignum-element-type half-bignum-element-type)
+  (values half-bignum-element-type half-bignum-element-type)
+  (foldable flushable movable always-translatable))
+
 ;;; Convert the input, a BIGNUM-ELEMENT-TYPE, to a signed word.
 ;;; FIXME: considering that both the input and output have N-WORD-BITS significant bits,
 ;;; this is really a bad name for the operation.
 (defknown %fixnum-digit-with-correct-sign (bignum-element-type) sb-vm:signed-word
-  (foldable flushable movable always-translatable))
+    (foldable flushable movable always-translatable))
 
 ;;; %ASHR- take a digit-size quantity and shift it to the left,
 ;;; returning a digit-size quantity.
@@ -804,3 +825,23 @@
 
 (defknown (%unary-truncate %unary-round) (real) integer
   (movable foldable flushable no-verify-arg-count))
+
+(defknown (%unary-floor %unary-ceiling) (real) integer
+  (movable foldable flushable))
+
+#+(or arm64 x86-64)
+(defknown sb-lockless::get-next (sb-lockless::list-node) (values sb-lockless::list-node t))
+
+(defknown sb-vm::fastrem-32 ((unsigned-byte 32) (unsigned-byte 32) (unsigned-byte 32))
+  (unsigned-byte 32)
+  (flushable))
+(defknown sb-vm::fastrem-64 ((unsigned-byte 64) (unsigned-byte 64) (unsigned-byte 64))
+  (unsigned-byte 64)
+  (flushable))
+
+;;; +-modfx is useful for computing a hash that is commutative in its inputs.
+;;; These architectures lack a complete set of modular operations; they have operations
+;;; which are modular in N-WORD-BITS, but not that accept fixnums.
+#+(or arm mips ppc sparc)
+(defknown sb-vm::+-modfx (integer integer) fixnum
+          (movable foldable flushable always-translatable))
