@@ -117,26 +117,15 @@ RETURN-FROM can be used to exit the form."
                                   :cleanup cleanup)))
       (ir1-convert-progn-body dummy next result forms))))
 
+;;; We make NEXT start a block just so that it will have a block
+;;; assigned. People assume that when they pass a ctran into
+;;; IR1-CONVERT as NEXT, it will have a block when it is done.
 (def-ir1-translator return-from ((name &optional value) start next result)
   "RETURN-FROM name value
 
 Evaluate the VALUE, returning its values from the lexically enclosing
 block NAME. This is constrained to be used only within the dynamic
 extent of the block."
-  ;; old comment:
-  ;;   We make NEXT start a block just so that it will have a block
-  ;;   assigned. People assume that when they pass a ctran into
-  ;;   IR1-CONVERT as NEXT, it will have a block when it is done.
-  ;; KLUDGE: Note that this block is basically fictitious. In the code
-  ;;   (BLOCK B (RETURN-FROM B) (SETQ X 3))
-  ;; it's the block which answers the question "which block is
-  ;; the (SETQ X 3) in?" when the right answer is that (SETQ X 3) is
-  ;; dead code and so doesn't really have a block at all. The existence
-  ;; of this block, and that way that it doesn't explicitly say
-  ;; "I'm actually nowhere at all" makes some logic (e.g.
-  ;; BLOCK-HOME-LAMBDA-OR-NULL) more obscure, and it might be better
-  ;; to get rid of it, perhaps using a special placeholder value
-  ;; to indicate the orphanedness of the code.
   (ctran-starts-block next)
   (let* ((found (or (lexenv-find name blocks)
                     (compiler-error "return for unknown block: ~S" name)))
@@ -602,9 +591,6 @@ Return VALUE without evaluating it."
          `(lambda ,(strip-lambda-list (third thing) :name) ,(name-context))))
     ((lambda)
      `(lambda ,(strip-lambda-list (second thing) :name) ,@(name-context)))
-    ((lambda-with-lexenv)
-     ;; FIXME: Get the original DEFUN name here.
-     `(lambda ,(fifth thing)))
     (otherwise
      (compiler-error "Not a valid lambda expression:~%  ~S"
                      thing))))
@@ -621,7 +607,7 @@ Return VALUE without evaluating it."
 ;;; apparent function associated to it.
 (defun find-or-convert-fun-leaf (thing start)
   (cond
-    ((typep thing '(cons (member lambda named-lambda lambda-with-lexenv)))
+    ((typep thing '(cons (member lambda named-lambda)))
      (let ((ctran (make-ctran))
            (leaf (ir1-convert-lambdalike thing
                                          :debug-name (name-lambdalike thing))))
@@ -746,7 +732,7 @@ be a lambda expression."
   (let ((ctran (make-ctran))
         (fun-lvar (make-lvar)))
     (ir1-convert start ctran fun-lvar `(the function ,function))
-    (ir1-convert-combination-args fun-lvar ctran next result args nil)))
+    (ir1-convert-combination-args fun-lvar ctran next result args :pass-nargs nil)))
 
 ;;; This source transform exists to reduce the amount of work for the
 ;;; compiler. If the called function is a FUNCTION form, then convert
@@ -850,7 +836,8 @@ have been evaluated."
                                    :debug-name (debug-name 'let bindings))))
                          (reference-leaf start ctran fun-lvar fun))
                        (values next result))))
-           (ir1-convert-combination-args fun-lvar ctran next result values)))))
+           (ir1-convert-combination-args fun-lvar ctran next result values
+                                         :arg-source-forms bindings)))))
 
 (def-ir1-translator let* ((bindings &body body)
                           start next result)
@@ -861,7 +848,8 @@ form to reference any of the previous VARS."
   (multiple-value-bind (vars values forms decls) (parse-letish bindings body 'let*)
     (processing-decls (decls vars nil next result post-binding-lexenv)
       (ir1-convert-aux-bindings
-       start next result forms vars values post-binding-lexenv))))
+       start next result forms vars values post-binding-lexenv
+       :value-source-forms bindings))))
 
 ;;; logic shared between IR1 translators for LOCALLY, MACROLET,
 ;;; and SYMBOL-MACROLET
@@ -942,9 +930,11 @@ also processed as top level forms."
     (enclose start enclose-ctran funs)
     (cond ((some #'leaf-dynamic-extent funs)
            (ctran-starts-block next)
-           (let ((cleanup (make-cleanup :kind :dynamic-extent
-                                        :mess-up (ctran-use enclose-ctran)))
-                 (cleanup-ctran (make-ctran)))
+           (let* ((enclose (ctran-use enclose-ctran))
+                  (cleanup (make-cleanup :kind :dynamic-extent
+                                         :mess-up enclose))
+                  (cleanup-ctran (make-ctran)))
+             (setf (enclose-cleanup enclose) cleanup)
              (let ((*lexenv* (make-lexenv :cleanup cleanup)))
                (ir1-convert enclose-ctran cleanup-ctran nil '(%cleanup-point))
                (ir1-convert-progn-body cleanup-ctran next result body))))
