@@ -112,12 +112,12 @@
                                  *current-path*))))
     (funcall thunk)))
 
-(defvar *derive-function-types* :same-file
-  "If true, argument and result type information derived from
-  compilation of DEFUNs is used when compiling calls to that
-  function. If :SAME-FILE (the default, as allowed by ANSI 3.2.2.3),
-  the information is derived only from DEFUNs in the same file. If
-  false, only information from FTYPE proclamations will be used.")
+(defvar *derive-function-types* nil
+  "Should the compiler assume that function types will never change,
+  so that it can use type information inferred from current definitions
+  to optimize code which uses those definitions? Setting this true
+  gives non-ANSI, early-CMU-CL behavior. It can be useful for improving
+  the efficiency of stable code.")
 
 ;;;; namespace management utilities
 
@@ -164,7 +164,13 @@
               ;; FIXME: this seems to omit FUNCTIONAL
               (when (defined-fun-p fun)
                 (return-from fun-lexically-notinline-p
-                  (eq (defined-fun-inlinep fun) 'notinline))))))))
+                  (eq (defined-fun-inlinep fun) 'notinline)))
+              (loop for data in (lexenv-user-data env)
+                    when (and (eq (car data) 'no-compiler-macro)
+                              (eq (cdr data) name))
+                    do
+                    (return-from fun-lexically-notinline-p
+                      t)))))))
     ;; If ANSWER is NIL, go for the global value
     (eq (or answer (info :function :inlinep name)) 'notinline)))
 
@@ -214,11 +220,10 @@
          :type (if (or (eq where :declared)
                        (and (not latep)
                             (not notinline)
-                            (eq *derive-function-types* t)))
+                            *derive-function-types*))
                    ftype
                    (specifier-type 'function))
-         :defined-type (if (and (not latep) (not notinline)
-                                *derive-function-types*)
+         :defined-type (if (and (not latep) (not notinline))
                            ftype
                            (specifier-type 'function))
          :where-from (if notinline
@@ -311,7 +316,13 @@
                        (type (type-specifier (info :variable :type name))))
                    `(macro . (the ,type ,expansion))))
                 (:constant
-                 (find-constant (symbol-value name) name))
+                 (let ((value (symbol-value name)))
+                   ;; Objects that are freely coalescible become
+                   ;; anonymous since we aren't interested in
+                   ;; accessing them through their name.
+                   (if (sb-xc:typep value '(or number character symbol))
+                       (find-constant value)
+                       (make-constant value (ctype-of value) name))))
                 (t
                  (make-global-var :kind kind
                                   :%source-name name
@@ -336,7 +347,8 @@
       (cl:typep obj 'package)
       ;; The cross-compiler wants to dump CTYPE instances as leaves,
       ;; but CLASSOIDs are excluded since they have a MAKE-LOAD-FORM method.
-      #+sb-xc-host (cl:typep obj '(and ctype (not classoid)))))
+      #+sb-xc-host (cl:typep obj '(and ctype (not classoid)))
+      (eq obj sb-lockless:+tail+)))
 
 ;;; Grovel over CONSTANT checking for any sub-parts that need to be
 ;;; processed with MAKE-LOAD-FORM. We have to be careful, because
@@ -544,8 +556,12 @@
       (declare (fixnum pos))
       (macrolet ((frob ()
                    `(progn
-                      (when (atom subform) (return))
-                      (let ((fm (car subform)))
+                      (let ((fm (cond ((comma-p subform)
+                                       (comma-expr subform))
+                                      ((atom subform)
+                                       (return))
+                                      (t
+                                       (car subform)))))
                         (when (comma-p fm)
                           (setf fm (comma-expr fm)))
                         (cond ((consp fm)
@@ -563,6 +579,8 @@
                                ;; perfect, but better than nothing.
                                (note-source-path subform pos path)))
                         (incf pos))
+                      (when (comma-p subform)
+                        (return))
                       (setq subform (cdr subform))
                       (when (eq subform trail) (return)))))
         (loop
@@ -970,7 +988,8 @@
 ;;; arguments for the call, which defaults to the cdr of source. We
 ;;; return the COMBINATION node.
 (defun ir1-convert-combination-args (fun-lvar start next result args
-                                     &optional (pass-nargs t))
+                                     &key (pass-nargs t)
+                                          arg-source-forms)
   (declare (type ctran start next)
            (type lvar fun-lvar)
            (type (or lvar null) result)
@@ -991,7 +1010,9 @@
                  (setf this-start
                        (maybe-instrument-progn-like this-start forms arg))
                  (let ((this-ctran (make-ctran))
-                       (this-lvar (make-lvar node)))
+                       (this-lvar (make-lvar node))
+                       (*current-path* (or (get-source-path (pop arg-source-forms))
+                                           *current-path*)))
                    (ir1-convert this-start this-ctran this-lvar arg)
                    (setq this-start this-ctran)
                    (arg-lvars this-lvar))))
@@ -1337,8 +1358,8 @@
     (when (defined-fun-p var)
       (setf (defined-fun-inline-expansion res)
             (defined-fun-inline-expansion var))
-      (setf (defined-fun-functionals res)
-            (defined-fun-functionals var))
+      (setf (defined-fun-functional res)
+            (defined-fun-functional var))
       (setf (defined-fun-same-block-p res)
             (defined-fun-same-block-p var)))
     ;; FIXME: Is this really right? Needs we not set the FUNCTIONAL
@@ -1459,10 +1480,10 @@ the stack without triggering overflow protection.")
 (defun process-extent-decl (names vars fvars kind)
   (let ((extent
           (ecase kind
-            (dynamic-extent
+            ((dynamic-extent dynamic-extent-no-note)
              (when *stack-allocate-dynamic-extent*
                kind))
-            ((indefinite-extent truly-dynamic-extent)
+            ((truly-dynamic-extent)
              kind))))
     (if extent
         (dolist (name names)
@@ -1478,9 +1499,10 @@ the stack without triggering overflow protection.")
                   (cond
                     ((and (typep var 'global-var) (eq (global-var-kind var) :unknown)))
                     (bound-var
-                     (if (and (leaf-extent var) (neq extent (leaf-extent var)))
+                     (if (and (leaf-dynamic-extent var)
+                              (neq extent (leaf-dynamic-extent var)))
                          (warn "Multiple incompatible extent declarations for ~S?" name)
-                         (setf (leaf-extent var) extent)))
+                         (setf (leaf-dynamic-extent var) extent)))
                     (t (compiler-notify "Ignoring free ~S declaration: ~S" kind name))))
                  (cons
                   (compiler-error "~S on symbol-macro: ~S" kind name))
@@ -1489,8 +1511,7 @@ the stack without triggering overflow protection.")
             ((and (consp name)
                   (eq (car name) 'function)
                   (null (cddr name))
-                  (valid-function-name-p (cadr name))
-                  (neq extent 'indefinite-extent))
+                  (valid-function-name-p (cadr name)))
              (let* ((fname (cadr name))
                     (bound-fun (find fname fvars
                                      :key (lambda (x)
@@ -1501,7 +1522,7 @@ the stack without triggering overflow protection.")
                (etypecase fun
                  (leaf
                   (if bound-fun
-                      (setf (leaf-extent bound-fun) extent)
+                      (setf (leaf-dynamic-extent bound-fun) extent)
                       (compiler-notify
                        "Ignoring free DYNAMIC-EXTENT declaration: ~S" name)))
                  (cons
@@ -1550,6 +1571,11 @@ the stack without triggering overflow protection.")
         (process-ftype-decl (second spec) res (cddr spec) fvars context))
        ((inline notinline maybe-inline)
         (process-inline-decl spec res fvars))
+       (no-compiler-macro
+        (make-lexenv :default res
+                     :user-data (list*
+                                 (cons 'no-compiler-macro (second spec))
+                                 (lexenv-user-data res))))
        (optimize
         (multiple-value-bind (new-policy specified-qualities)
             (process-optimize-decl spec (lexenv-policy res))
@@ -1565,7 +1591,7 @@ the stack without triggering overflow protection.")
          :default res
          :handled-conditions (process-unmuffle-conditions-decl
                               spec (lexenv-handled-conditions res))))
-       ((dynamic-extent truly-dynamic-extent indefinite-extent)
+       ((dynamic-extent truly-dynamic-extent dynamic-extent-no-note)
         (process-extent-decl (cdr spec) vars fvars (first spec))
         res)
        ((disable-package-locks enable-package-locks)
@@ -1681,6 +1707,7 @@ the stack without triggering overflow protection.")
                                       it))
                     (setq explicit-check (or (cdr spec) t)
                           allow-explicit-check nil)) ; at most one of this decl
+                   ((equal spec '(top-level-form))) ; ignore
                    ((typep spec '(cons (eql source-form)))
                     (setf source-form (cadr spec)))
                    ;; Used only for the current function.
@@ -1810,7 +1837,7 @@ the stack without triggering overflow protection.")
                     :inlinep (defined-fun-inlinep var)
                     :inline-expansion (defined-fun-inline-expansion var)
                     :same-block-p (defined-fun-same-block-p var)
-                    :functionals (defined-fun-functionals var))
+                    :functional (defined-fun-functional var))
                    (make-global-var :%source-name name :type type
                                     :where-from :declared :kind kind))))
        (when (defined-fun-p var)
