@@ -380,7 +380,7 @@
 ;;; methods are passed an additional POLICY argument, and IR2-CONVERT
 ;;; methods are passed an additional IR2-BLOCK argument.
 (defmacro defoptimizer (what (lambda-list
-                              &optional (node (sb-xc:gensym) node-p)
+                              &optional (node (gensym))
                               &rest vars)
                         &body body)
   (let ((name (flet ((function-name (name)
@@ -402,37 +402,55 @@
              ,@body)
            ,@(loop for vop-name in (ensure-list (second what))
                    collect
-                   `(set-vop-optimizer (template-or-lose ',vop-name) #',name)))
+                   `(set-vop-optimizer (template-or-lose ',vop-name)
+                                       ,(if (cddr what)
+                                            `(cons #',name ',(caddr what))
+                                            `#',name))))
         (binding* (((forms decls) (parse-body body nil))
                    ((var-decls more-decls) (extract-var-decls decls vars))
-                   ;; In case the BODY declares IGNORE of the formal NODE var,
-                   ;; we rebind it from N-NODE and never reference it from BINDS.
-                   (n-node (make-symbol "NODE"))
-                   ((binds lambda-vars gensyms)
-                    (parse-deftransform lambda-list n-node
+                   ((binds lambda-vars)
+                    (parse-deftransform lambda-list node
                                         `(return-from ,name
                                            ,(if (and (consp what)
                                                      (eq (second what)
                                                          'equality-constraint))
                                                 :give-up
-                                                nil)))))
+                                                nil))))
+                   (args (gensym)))
           (declare (ignore lambda-vars))
           `(progn
              ;; We can't stuff the BINDS as &AUX vars into the lambda list
              ;; because there can be a RETURN-FROM in there.
-             (defun ,name (,n-node ,@vars)
+             (defun ,name (,node ,@vars &rest ,args)
+               (declare (ignorable ,node ,@(butlast vars))
+                        (ignore ,args))
                ,@(if var-decls (list var-decls))
-               (let* (,@binds ,@(if node-p `((,node ,n-node))))
-                 ;; Syntax requires naming NODE even if undesired if VARS
-                 ;; are present, so in that case make NODE ignorable.
-                 (declare (ignorable ,@(if (and vars node-p) `(,node))
-                                     ,@gensyms))
+               (let* (,@binds)
+                 (declare (ignorable ,@(mapcar #'car binds)))
                  ,@more-decls ,@forms))
              ,@(when (consp what)
                  `((setf (,(let ((*package* (sb-xc:symbol-package 'fun-info)))
                              (symbolicate "FUN-INFO-" (second what)))
                           (fun-info-or-lose ',(first what)))
                          #',name))))))))
+
+(defmacro defoptimizers (kind names (lambda-list
+                                     &optional (node (gensym))
+                                     &rest vars)
+                         &body body)
+  (let ((optimizer-name (symbolicate (car names)
+                                     "-"
+                                     kind
+                                     "-OPTIMIZER")))
+    `(progn
+       (defoptimizer ,optimizer-name
+           (,lambda-list ,node ,@vars)
+         ,@body)
+       ,@(loop for name in names
+               collect `(setf (,(let ((*package* (sb-xc:symbol-package 'fun-info)))
+                                  (symbolicate "FUN-INFO-" kind))
+                               (fun-info-or-lose ',name))
+                              #',optimizer-name)))))
 
 ;;;; IR groveling macros
 
@@ -448,7 +466,7 @@
 (defmacro do-blocks ((block-var component &optional ends result) &body body)
   (unless (member ends '(nil :head :tail :both))
     (error "losing ENDS value: ~S" ends))
-  (let ((n-component (sb-xc:gensym))
+  (let ((n-component (gensym))
         (n-tail (gensym)))
     `(let* ((,n-component ,component)
             (,n-tail ,(if (member ends '(:both :tail))
@@ -464,7 +482,7 @@
 (defmacro do-blocks-backwards ((block-var component &optional ends result) &body body)
   (unless (member ends '(nil :head :tail :both))
     (error "losing ENDS value: ~S" ends))
-  (let ((n-component (sb-xc:gensym))
+  (let ((n-component (gensym))
         (n-head (gensym)))
     `(let* ((,n-component ,component)
             (,n-head ,(if (member ends '(:both :head))
@@ -572,21 +590,19 @@
            while ,node-var
            do (progn ,@body))))
 
-(defmacro do-nested-cleanups ((cleanup-var lexenv &optional return-value)
+(defmacro do-nested-cleanups ((cleanup-var block &optional return-value)
                               &body body)
   `(block nil
      (map-nested-cleanups
-      (lambda (,cleanup-var) ,@body) ,lexenv ,return-value)))
+      (lambda (,cleanup-var) ,@body) ,block ,return-value)))
 
 ;;; Bind the IR1 context variables to the values associated with NODE,
 ;;; so that new, extra IR1 conversion related to NODE can be done
 ;;; after the original conversion pass has finished.
 (defmacro with-ir1-environment-from-node (node &rest forms)
-  `(flet ((closure-needing-ir1-environment-from-node ()
-            ,@forms))
-     (%with-ir1-environment-from-node
-      ,node
-      #'closure-needing-ir1-environment-from-node)))
+  `(%with-ir1-environment-from-node
+    ,node
+    (lambda () ,@forms)))
 
 ;;; *SOURCE-PATHS* is a hashtable from source code forms to the path
 ;;; taken through the source to reach the form. This provides a way to
@@ -695,8 +711,8 @@
               #-sb-xc-host get-setf-expansion place env)
            (when (cdr stores)
              (error "multiple store variables for ~S" place))
-           (let ((newval (sb-xc:gensym))
-                 (n-place (sb-xc:gensym))
+           (let ((newval (gensym))
+                 (n-place (gensym))
                  (mask (encode-attribute-mask attributes ,vector)))
              (values `(,@temps ,n-place)
                      `(,@values ,getter)
