@@ -39,7 +39,7 @@
   (:results)
   (:vop-var vop)
   (:node-var node)
-  (:args-var args)
+  (:arg-refs args)
   (:temporary (:sc unsigned-reg) val-temp)
   (:generator 1
     (cond #+ubsan
@@ -428,7 +428,7 @@
   (:args (symbol :scs (descriptor-reg)))
   (:results (res :scs (any-reg)))
   (:result-types positive-fixnum)
-  (:args-var args)
+  (:arg-refs args)
   (:generator 2
     (loadw res symbol symbol-hash-slot other-pointer-lowtag)
     ;; The symbol-hash slot of NIL holds NIL because it is also the
@@ -492,15 +492,16 @@
     (inst mov res (ea n-word-bytes base-ptr)) ; 1 word beyond the header
     (inst and res (lognot fixnum-tag-mask))))
 
+(aver (= sb-impl::package-id-bits 16))
 (define-vop ()
   (:args (symbol :scs (descriptor-reg)))
   (:results (result :scs (unsigned-reg)))
   (:result-types positive-fixnum)
-  (:translate sb-impl::symbol-package-id)
+  (:translate symbol-package-id)
   (:policy :fast-safe)
-  (:generator 2 ; ASSUMPTION: symbol-package-bits = 16
-   (inst mov :word result (object-slot-ea symbol symbol-name-slot (- other-pointer-lowtag 6)))
-   (inst movzx '(:word :dword) result result)))
+  (:generator 2
+   (inst movzx '(:word :dword) result
+         (object-slot-ea symbol symbol-name-slot (- other-pointer-lowtag 6)))))
 (define-vop ()
   (:args (symbol :scs (descriptor-reg)))
   (:results (result :scs (descriptor-reg) :from :load))
@@ -580,6 +581,7 @@
   (:policy :fast-safe)
   (:translate fdefn-makunbound)
   (:args (fdefn :scs (descriptor-reg)))
+  (:temporary (:sc unsigned-reg) temp)
   (:vop-var vop)
   (:generator 38
     ;; Change the JMP instruction to INT3 so that a trap occurs in the fdefn
@@ -595,8 +597,9 @@
     ;; and resume at undefined-tramp. However, CALL-SYMBOL jumps via raw-addr if
     ;; its callable object was not a function. In that case RAX holds a symbol,
     ;; so we're OK because we can identify the undefined function.
-    (storew (make-fixup 'undefined-tramp :assembly-routine)
-            fdefn fdefn-raw-addr-slot other-pointer-lowtag)))
+    ;; Technically this could be computed using "LEA temp, [RIP-disp]"
+    (inst mov temp (ea (make-fixup 'undefined-tramp :assembly-routine*)))
+    (storew temp fdefn fdefn-raw-addr-slot other-pointer-lowtag)))
 
 ;;;; binding and unbinding
 
