@@ -83,6 +83,9 @@
 (defun reg-imm-data (dchunk dstate) dchunk
   (aref (sb-disassem::dstate-filtered-values dstate) 4))
 
+;;; This structure is logically immutable, except for one problem:
+;;; the disassembler recycles instances of it (re-uses the same
+;;; one for each successive instruction). See DECODE-MOD-R/M.
 (defstruct (machine-ea (:copier nil)
                        (:constructor %make-machine-ea))
   ;; possible TODO: base,index,scale could be packed thusly in 13 bits:
@@ -174,7 +177,25 @@
    value (inst-operand-size-default-qword dstate) t stream dstate))
 
 (defun print-jmp-ea (value stream dstate)
-  (print-sized-reg/mem-default-qword value stream dstate))
+  (cond ((typep value 'machine-ea)
+         (print-mem-ref :ref value :qword stream dstate)
+         #+immobile-space
+         (when (and (null (machine-ea-base value))
+                    (null (machine-ea-index value)))
+           (let* ((v (- (sb-vm::static-call-entrypoint-vector) other-pointer-lowtag))
+                  (data (+ v (ash sb-vm:vector-data-offset sb-vm:word-shift)))
+                  (end (+ data (ash (length +static-fdefns+) sb-vm:word-shift))))
+             (when (<= data (machine-ea-disp value) (1- end))
+               (let ((i (ash (- (machine-ea-disp value) data) (- sb-vm:word-shift))))
+                 (note (lambda (stream) (prin1 (aref +static-fdefns+ i) stream)) dstate)
+                 (return-from print-jmp-ea)))
+             (let* ((v sb-fasl::*asm-routine-vector*)
+                    (a (logandc2 (get-lisp-obj-address v) sb-vm:lowtag-mask)))
+               (when (<= a (machine-ea-disp value) (1- (+ a (primitive-object-size v))))
+                 (let ((target (sap-ref-word (int-sap (machine-ea-disp value)) 0)))
+                   (maybe-note-assembler-routine target t dstate)))))))
+        ((null stream) (operand value dstate))
+        (t (write value :stream stream))))
 
 (defun print-sized-byte-reg/mem (value stream dstate)
   (print-reg/mem-with-width value :byte t stream dstate))
@@ -274,18 +295,6 @@
               (sb-disassem::find-code-constant-from-interior-pointer value dstate))
     (note (lambda (stream) (princ it stream)) dstate)))
 
-;;; Immobile symbols are heuristically detected with MAYBE-NOTE-STATIC-LISPOBJ
-;;; so that moving the tagged pointer to a symbol is shown thusly:
-;;; ";  BACF003C50       MOV EDX, #x503C00CF             ; '*MYVAR*"
-;;;
-(defun print-imm/asm-routine (value stream dstate)
-  (cond ((not stream) (operand value dstate))
-        ((or (maybe-note-assembler-routine value nil dstate)
-             (maybe-note-static-lispobj value dstate t))
-         (princ16 value stream))
-        (t
-         (princ value stream))))
-
 ;;; Return an instance of REG or MACHINE-EA.
 ;;; MOD and R/M are the extracted bits from the instruction's ModRM byte.
 ;;; Depending on MOD and R/M, a SIB byte and/or displacement may be read.
@@ -440,7 +449,13 @@
          ;; compilation to memory says it is all associated with
          ;; the symbol "lisp_jit_code" which is not useful.
          (when (plusp addr)
-           (or (unless (sb-kernel:immobile-space-addr-p addr)
+           (or (when (<= sb-vm:alien-linkage-table-space-start addr
+                         (+ sb-vm:alien-linkage-table-space-start
+                            (1- sb-vm:alien-linkage-table-space-size)))
+                 (let* ((index (sb-vm::alien-linkage-table-index-from-address addr))
+                        (name (sb-impl::alien-linkage-index-to-name index)))
+                   (note (lambda (s) (format s "&~A" name)) dstate)))
+               (unless (sb-kernel:immobile-space-addr-p addr)
                  (maybe-note-assembler-routine addr nil dstate))
                ;; Show the absolute address and maybe the contents.
                (note (format nil "[#x~x]~@[ = #x~x~]"
@@ -449,6 +464,19 @@
                               (:qword (unboxed-constant-ref dstate addr disp))))
                      dstate))))))
 
+    ;; Recognize "[Rbase+disp]" as an alien linkage table reference if Rbase was
+    ;; just loaded with the base address in the prior instruction.
+    (when (and (eql (machine-ea-base value)
+                    (car (sb-disassem::dstate-known-register-contents dstate)))
+               (eq (cdr (sb-disassem::dstate-known-register-contents dstate))
+                   'alien-linkage)
+               (not (machine-ea-index value))
+               (integerp (machine-ea-disp value)))
+      (let ((name (sb-impl::alien-linkage-index-to-name
+                   (floor (machine-ea-disp value) sb-vm:alien-linkage-table-entry-size))))
+        (note (lambda (s) (format s "&~A" name)) dstate)))
+    (setf (sb-disassem::dstate-known-register-contents dstate) nil)
+
     (flet ((guess-symbol (predicate)
              (binding* ((code-header (seg-code (dstate-segment dstate)) :exit-if-null)
                         (header-n-words (code-header-words code-header)))
@@ -481,14 +509,24 @@
                   #+gs-seg (dstate-getprop dstate +gs-segment+)
                   #-gs-seg (not (dstate-getprop dstate +fs-segment+)) ; not system TLS
                   (not index-reg) ; no index
-                  (typep disp '(integer 0 *)) ; positive displacement
+                  (typep disp '(integer -128 *)) ; valid displacement
                   (zerop (logand disp 7))) ; lispword-aligned
-             (let ((index (ash disp -3)))
-               (when (< index (length thread-slot-names))
-                 (awhen (aref thread-slot-names index)
-                   (return-from print-mem-ref
-                     (note (lambda (stream) (format stream "thread.~(~A~)" it))
-                           dstate)))))
+             (let* ((index (ash disp -3))
+                    (symbol (cond ((minusp index)
+                                   (let ((index (1- (- index))))
+                                     (when (array-in-bounds-p sb-vm::+thread-header-slot-names+ index)
+                                       (aref sb-vm::+thread-header-slot-names+ index))))
+                                  ((< index (length thread-slot-names))
+                                   (aref thread-slot-names index)))))
+               (when symbol
+                 (when (and (eq symbol 'sb-vm::alien-linkage-table-base)
+                            (eql (logandc2 (sb-disassem::dstate-inst-properties dstate) +rex-r+)
+                                 (logior +rex+ +rex-w+ +rex-b+)))
+                   (setf (sb-disassem::dstate-known-register-contents dstate)
+                         `(,(reg-num (regrm-inst-reg dchunk-zero dstate)) . alien-linkage)))
+                 (return-from print-mem-ref
+                   (note (lambda (stream) (format stream "thread.~(~A~)" symbol))
+                         dstate))))
              (let ((symbol (or (guess-symbol
                                 (lambda (s) (= (symbol-tls-index s) disp)))
                                ;; static symbols aren't in the code header
@@ -511,41 +549,42 @@
 ;; Figure out whether LEA should print its EA with just the stuff in brackets,
 ;; or additionally show the EA as either a label or a hex literal.
 (defun lea-print-ea (value stream dstate)
-  (let* ((width (inst-operand-size dstate))
-         (addr
-          (etypecase value
-           (machine-ea
-            ;; Indicate to PRINT-MEM-REF that this is not a memory access.
-            (print-mem-ref :compute value width stream dstate)
-            (when (eq (machine-ea-base value) :rip)
-              (+ (dstate-next-addr dstate) (machine-ea-disp value))))
+  (let*
+      ((width (inst-operand-size dstate))
+       (addr
+        (etypecase value
+          (machine-ea
+           ;; Indicate to PRINT-MEM-REF that this is not a memory access.
+           (print-mem-ref :compute value width stream dstate)
+           (when (eq (machine-ea-base value) :rip)
+             (+ (dstate-next-addr dstate) (machine-ea-disp value))))
 
-           ((or string integer)
-            ;; A label for the EA should not print as itself, but as the decomposed
-            ;; addressing mode so that [ADDR] and [RIP+disp] are unmistakable.
-            ;; We can see an INTEGER here because LEA-COMPUTE-LABEL is always called
-            ;; on the operand to LEA, and it will compute an absolute address based
-            ;; off RIP when possible. If :use-labels NIL was specified, there is
-            ;; no hashtable of address to string, so we get the address.
-            ;; But ordinarily we get the string. Either way, the r/m arg reveals the
-            ;; EA calculation. DCHUNK-ZERO is a meaningless value - any would do -
-            ;; because the EA was computed in a prefilter.
-            ;; (the instruction format is known because LEA has exactly one format)
-            (print-mem-ref :compute (regrm-inst-r/m dchunk-zero dstate)
-                           width stream dstate)
-            value)
+          ((or string integer)
+           ;; A label for the EA should not print as itself, but as the decomposed
+           ;; addressing mode so that [ADDR] and [RIP+disp] are unmistakable.
+           ;; We can see an INTEGER here because LEA-COMPUTE-LABEL is always called
+           ;; on the operand to LEA, and it will compute an absolute address based
+           ;; off RIP when possible. If :use-labels NIL was specified, there is
+           ;; no hashtable of address to string, so we get the address.
+           ;; But ordinarily we get the string. Either way, the r/m arg reveals the
+           ;; EA calculation. DCHUNK-ZERO is a meaningless value - any would do -
+           ;; because the EA was computed in a prefilter.
+           ;; (the instruction format is known because LEA has exactly one format)
+           (print-mem-ref :compute (regrm-inst-r/m dchunk-zero dstate)
+                          width stream dstate)
+           value)
 
-           ;; LEA Rx,Ry is an illegal encoding, but we'll show it as-is.
-           ;; When we used integers instead of REG to represent registers, this case
-           ;; overlapped with the preceding. It's nice that it no longer does.
-           (reg
-            (print-reg-with-width value width stream dstate)
-            nil))))
-
-    (when (and addr stream) ; no end-of-line comments if storing into dstate
-      (unless (maybe-note-assembler-routine addr nil dstate)
-        (note (lambda (s) (format s (if (stringp addr) "= ~A" "= #x~x") addr))
-              dstate)))))
+         ;; LEA Rx,Ry is an illegal encoding, but we'll show it as-is.
+         ;; When we used integers instead of REG to represent registers, this case
+         ;; overlapped with the preceding. It's nice that it no longer does.
+         (reg
+          (print-reg-with-width value width stream dstate)
+          nil))))
+    (when stream
+      (cond ((stringp addr) ; label
+             (note (lambda (s) (format s "= ~A" addr)) dstate))
+            (addr
+             (note (lambda (s) (format s "= #x~x" addr)) dstate))))))
 
 ;;;; interrupt instructions
 
@@ -602,6 +641,7 @@
          (jmp-inst (find-inst #xE9 inst-space))
          (cond-jmp-inst (find-inst #x800f inst-space))
          (lea-inst (find-inst #x8D inst-space))
+         (mov-inst (find-inst #x8B inst-space))
          (address (get-lisp-obj-address code))
          (text-start (sap-int (code-instructions code)))
          (text-end (+ text-start (%code-text-size code)))
@@ -610,7 +650,7 @@
           (seg-length segment) length
           (seg-sap-maker segment) (lambda () sap))
     (map-segment-instructions
-     (lambda (dchunk inst)
+     (lambda (dchunk inst &aux (opcode (sap-ref-8 sap (dstate-cur-offs dstate))))
        (flet ((includep (target)
                 ;; Self-relative (to the code object) operands are ignored.
                 (and (or (< target address) (>= target text-end))
@@ -629,10 +669,11 @@
                   (when (includep operand)
                     (funcall function (+ (dstate-cur-offs dstate) 2)
                              operand inst))))
-               ((eq inst lea-inst)
+               ((or (eq inst lea-inst)
+                    (and (eq inst mov-inst) (eql opcode #x8B)))
                 ;; Computing the address of UNDEFINED-FDEFN and
                 ;; FUNCALLABLE-INSTANCE-TRAMP is done with LEA.
-                (aver (eql (sap-ref-8 sap (dstate-cur-offs dstate)) #x8D))
+                ;; Load from the alien linkage table can be done with MOV Rnn,[RIP-k].
                 (let ((modrm (sap-ref-8 sap (1+ (dstate-cur-offs dstate)))))
                   (when (= (logand modrm #b11000111) #b00000101) ; RIP-relative mode
                     (let ((operand (+ (signed-sap-ref-32
