@@ -53,7 +53,7 @@
   ;; with a block name, or a BLOCK and some bindings.
   (def-subtype lambda-env (block)))
 
-;;; This implementation of lexical environments takes a fairly unconvential
+;;; This implementation of lexical environments takes a fairly unconventional
 ;;; approach to dealing with sequential (LET*) binding. [Anything said here
 ;;; about LET* pertains to LAMBDA as well]. The conceptual model for LET* is
 ;;; that it is a sequence of nested LET forms, each creating one binding.
@@ -120,7 +120,7 @@
 ;;; freezing must do a non-trivial amount of work are quite rare.
 
 ;;; All that said, the representation of "mutable" is that instead of
-;;; a vector of sybols, we have a cons of a fixnum and a vector, so ...
+;;; a vector of symbols, we have a cons of a fixnum and a vector, so ...
 ;;; Return T if the ENV is currently undergoing sequential variable binding.
 (declaim (inline env-mutable-p))
 (defun env-mutable-p (env) (consp (env-symbols env)))
@@ -163,42 +163,12 @@
 ;;; "scope" and "frame" are basically synonymous here.
 ;;; The existence of both terms is a minor accident.
 
-;; Binding frame specification for LET and LET*
-;; This is a prototype stack-frame rather than a runtime stack frame in that
-;; one exists per syntactic form, not per dynamic invocation of same.
-(defstruct (frame (:include decl-scope)
-                  (:copier nil) (:predicate nil)
-                  (:constructor make-let-frame
-                                (declarations %policy
-                                 symbols special-b values sexpr specials)))
-  ;; If more symbols exist than values, the remainder are free specials.
-  (symbols    nil :read-only t :type simple-vector)
-  ;; Bitmask over symbols. 1 in bit N means bind the Nth symbol as special.
-  (special-b  nil :read-only t :type integer)
-  ;; A let frame can't have zero values. (It would be converted to LOCALLY)
-  (values nil :read-only t :type simple-vector)
-  (sexpr      nil :read-only t) ; code to execute
-  ;; To avoid reconstituting the first PROGV operand from the special-b mask
-  ;; and vector of all bound symbols, store the bound specials as follows:
-  ;;   for LET    - a list of all bound specials
-  ;;   for LET*   - a list of singleton lists of bound specials
-  ;;   for LAMBDA - possibly both of the preceding: a list for mandatory args
-  ;;                and lists of singleton lists for &optional/&rest/&key.
-  (specials   nil :read-only t))
-
 (defmethod print-object ((self frame) stream)
   (print-unreadable-object (self stream :type t :identity t)))
 
 (declaim (inline frame-size))
 (defun frame-size (frame) (length (frame-values frame)))
 
-;;; A LET* frame is just like a LET frame.
-(defstruct (let*-frame
-             (:include frame) (:predicate nil) (:copier nil)
-             (:constructor make-let*-frame
-                           (declarations %policy symbols special-b
-                            values sexpr specials))))
-
 ;;; BASIC-ENV stores a policy for its body, but if evaluation has not reached
 ;;; the body forms of a LET*, then the old policy is in effect. This is due to
 ;;; the need for presenting a consistent view of the policy, but a frozen ENV
@@ -215,30 +185,6 @@
                  (env-policy (env-parent env))
                  policy)))))
 
-;; Fancy binding frame specification
-(defstruct (lambda-frame (:include let*-frame) (:predicate nil) (:copier nil))
-  ;; Unlike for a LET* frame the count of bound values can not be determined
-  ;; from the length of the VALUES vector, which contains various extra markers
-  ;; dictating how the arguments are to be parsed.
-  (n-bound-vars  0   :read-only t :type fixnum)
-  ;; Number of mandatory and optional arguments.
-  (min-args      0   :read-only t :type fixnum)
-  (n-optional    0   :read-only t :type fixnum)
-  ;; Packed flags indicating presence of &REST/&KEY/&ALLOW-OTHER-KEYS.
-  (keyword-bits  0   :read-only t :type fixnum)
-  ;; A BLOCK name in which to wrap the lambda's evaluable forms.
-  ;; This behaves exactly the same as using a block-env around the forms,
-  ;; however a lambda-env consumes only 8 words plus 2 for the freshly consed
-  ;; catch tag; whereas a var-env + block-env would consume 6 + 6 + 2, which
-  ;; entails 40% more overhead to enter the most trivial interpreted function.
-  (block-name    0   :read-only t :type (or (eql 0) symbol))
-  ;; SHARE-BLOCK-P is T if BLOCK-NAME can be created concurrently
-  ;; with variable bindings. If NIL when a block-name is present,
-  ;; then another environment is allocated to enclose the block,
-  ;; which is not as big a win as combining the block-env and var-env,
-  ;; but still beneficial as it avoids separately calling the block handler.
-  (share-block-p nil :read-only t :type boolean))
-
 (defconstant +restp-bit+  #b100)
 (defconstant +keyp-bit+   #b010)
 (defconstant +allowp-bit+ #b001)
@@ -270,7 +216,7 @@
                            (recurse it (and globalp (not (lambda-env-p env)))))
                           (globalp
                            (member symbol sb-c::*disabled-package-locks*))))
-      ;; Ambigous case: both in the same decl list. Oh well...
+      ;; Ambiguous case: both in the same decl list. Oh well...
       (case (car declaration)
         (disable-package-locks
          (when (member symbol (cdr declaration)) (return t)))
@@ -346,7 +292,8 @@
 (defmacro with-environment-vars ((symbols end) env &body body)
   `(awhen (env-symbols ,env)
      (let ((,symbols (truly-the simple-vector (if (listp it) (cdr it) it)))
-           (,end (locally (declare (optimize (safety 0))) (car it))))
+           (,end #+ubsan (if (listp it) (car it) (sb-c::vector-length it))
+                 #-ubsan (locally (declare (optimize (safety 0))) (car it))))
        (declare (index-or-minus-1 ,end))
        ,@body)))
 (eval-when (:compile-toplevel)
@@ -547,7 +494,7 @@
       (values (proto-fn-%frame proto-fn) (proto-fn-cookie proto-fn))
       (digest-lambda env proto-fn)))
 
-(defun %fun-type (fun)
+(defun %fun-ftype (fun)
   (let ((proto-fn (fun-proto-fn fun)))
     (or (proto-fn-type proto-fn)
         (setf (proto-fn-type proto-fn)
@@ -678,7 +625,7 @@
             (values 0 lambda-list body)))
     ;; Choke now if the list can't be parsed.
     ;; If lexical environment is NIL, :silent will be passed as NIL,
-    ;; and we can warn about "suspcious variables" and such.
+    ;; and we can warn about "suspicious variables" and such.
     (parse-lambda-list lambda-list :silent silent)
     (multiple-value-bind (forms decls docstring) (parse-body body t t)
       (%make-proto-fn name lambda-list decls forms docstring
@@ -893,7 +840,7 @@
   ;; then insert assertions for all variables bound by this scope.
   (when (and (policy env (>= safety 1))
              (find-if #'cdr symbols :end n-var-bindings))
-    (let ((checks (make-array n-var-bindings)))
+    (let ((checks (make-array n-var-bindings :initial-element 0)))
       (dotimes (i n-var-bindings (setf (binding-typechecks decl-scope) checks))
         (awhen (cdr (svref symbols i))
           (setf (svref checks i) (type-checker it))))))
@@ -1143,7 +1090,9 @@
                          (specialize binding))
                         ((eq reason 'compile)
                          ;; access interpreter's lexical vars
-                         (macroize sym `(svref ,payload ,i)))
+                         ;; Prevent SETF on the variable from getting
+                         ;; "Destructive function (SETF SVREF) called on constant data"
+                         (macroize sym `(svref (load-time-value ,payload) ,i)))
                         (t
                          (let ((leaf (make-lambda-var
                                       :%source-name sym
@@ -1190,9 +1139,7 @@
                    (setf (sb-c::lambda-var-flags var) 1)))))
             ((inline notinline)
              ;; This is just enough to get sb-cltl2 tests to pass.
-             (let ((inlinep (case (car spec)
-                              (inline :inline)
-                              (notinline :notinline))))
+             (let ((inlinep (car spec)))
                (dolist (fname (cdr spec))
                  (let ((fun (cdr (assoc fname funs :test 'equal))))
                    (typecase fun
@@ -1216,8 +1163,8 @@
             ((ignorable type optimize special dynamic-extent)
              )
             (t
-             (let ((fn (info :declaration :handler (first spec))))
-               (when fn
+             (let ((fn (info :declaration :known (first spec))))
+               (when (functionp fn)
                  (setq lexenv
                        (funcall
                         fn lexenv spec
@@ -1249,31 +1196,26 @@
               )))
     lexenv))
 
-;;; Produce the source representation expected by :INLINE-EXPANSION-DESIGNATOR.
-(defun reconstruct-syntactic-closure-env (env &aux guts)
-  (loop
-    (awhen (env-declarations env)
-      (setq guts `((:declare ,(apply 'append (mapcar 'cdr it)) ,@guts))))
-    (multiple-value-bind (kind data)
-        (typecase env
-          (macro-env
-           (values :macro
-                   (map 'list
-                        (lambda (f)
-                          ;; The name of each macro is (MACROLET symbol).
-                          (cons (second (fun-name f))
-                                (fun-lambda-expression f)))
-                        (env-payload env))))
-          (symbol-macro-env
-           (values :symbol-macro
-                   (map 'list (lambda (x y) (list (car x) y))
-                        (env-symbols env) (env-payload env)))))
-      (when kind
-        (setq guts `((,kind ,data ,@guts)))))
-    (unless (setq env (env-parent env))
-      (return (car guts)))))
+(defun inline-syntactic-closure-lambda (lambda env)
+  (labels ((frob (env decls)
+             (unless (or (basic-env-p env)
+                         (macro-env-p env)
+                         (symbol-macro-env-p env))
+               (return-from inline-syntactic-closure-lambda nil))
+             (let ((parent (env-parent env)))
+               (dolist (decl-spec (env-declarations env))
+                 (dolist (decl (cdr decl-spec))
+                   (push decl decls)))
+               (if parent
+                   (frob parent decls)
+                   decls))))
+    (let ((decls (frob env '()))
+          (expansion (sb-walker:macroexpand-all lambda env)))
+      (if decls
+          `(sb-c::lambda-with-lexenv ((declare ,@decls)) ,@(cdr expansion))
+          expansion))))
 
-;;; Return :INLINE or :NOTINLINE if FNAME has a lexical declaration,
+;;; Return INLINE or NOTINLINE if FNAME has a lexical declaration,
 ;;; otherwise NIL for no information.
 ;;; FIXME: obviously this does nothing
 (defun fun-lexically-notinline-p (fname env)
