@@ -57,7 +57,8 @@ lispobj sbcl_new_arena(size_t size)
     //   Memblk
     struct arena* arena = ARENA_GET_OS_MEMORY(size);
     memset(arena, 0, sizeof *arena);
-    arena->header = (sizeof (struct arena) / N_WORD_BYTES) << INSTANCE_LENGTH_SHIFT;
+    arena->header = ((sizeof (struct arena) / N_WORD_BYTES) << INSTANCE_LENGTH_SHIFT)
+      | INSTANCE_WIDETAG;
     struct arena_memblk* block =
       (void*)((char*)arena + ALIGN_UP(sizeof (struct arena), 2*N_WORD_BYTES));
     // arenas require threads, but the header for the mutex definition
@@ -103,6 +104,11 @@ static inline void* arena_mutex(struct arena* a) {
 void arena_release_memblks(lispobj arena_taggedptr)
 {
     struct arena* arena = (void*)native_pointer(arena_taggedptr);
+    // Avoid a race with GC in scavenge_arenas(). We're messing up the chain
+    // of blocks and can't have anything else looking at them.
+    __attribute__((unused)) lispobj old_hidden
+      = __sync_val_compare_and_swap(&arena->hidden, NIL, LISP_T);
+    gc_assert(old_hidden == NIL);
     ARENA_MUTEX_ACQUIRE(arena);
     struct arena_memblk* first = (void*)arena->uw_first_block;
     struct arena_memblk* block = first->next;
@@ -115,8 +121,16 @@ void arena_release_memblks(lispobj arena_taggedptr)
     char* mem_base = (char*)first + sizeof (struct arena_memblk);
     first->freeptr = PTR_ALIGN_UP(mem_base, CHUNK_ALIGN);
     first->next = NULL;
-    arena->uw_extension_count = 0;
+    // Release huge-object blocks
+    block = (void*)arena->uw_huge_objects;
+    while (block) {
+        struct arena_memblk* next = block->next;
+        free(block);
+        block = next;
+    }
+    arena->uw_huge_objects = 0;
     arena->uw_length = arena->uw_original_size;
+    arena->hidden = NIL;
     ARENA_MUTEX_RELEASE(arena);
 }
 
@@ -178,6 +192,7 @@ void AMD64_SYSV_ABI switch_to_arena(lispobj arena_taggedptr,
         if (th->mixed_tlab.start_addr) gc_close_region(&th->mixed_tlab, PAGE_TYPE_MIXED);
         if (th->cons_tlab.start_addr) gc_close_region(&th->cons_tlab, PAGE_TYPE_CONS);
         release_gc_page_table_lock();
+#if 0 // this causes a data race, the very thing it's trying to avoid
         int arena_index = fixnum_value(arena->index);
         /* If this thread has potentially used this arena previously, see if
          * the TLAB pointers can be restored based on token validity */
@@ -190,6 +205,7 @@ void AMD64_SYSV_ABI switch_to_arena(lispobj arena_taggedptr,
             }
             memset(state, 0, sizeof (arena_state));
         }
+#endif
     } else { // finished with the arena
         gc_assert(th->arena); // must have been an arena in use
         struct arena* old_arena = (void*)native_pointer(th->arena);
@@ -233,28 +249,63 @@ static void* memblk_claim_subrange(struct arena* a, struct arena_memblk* mem,
             if (mem != actual_current_block) { // oops - looking at the wrong memblk
                 mem = actual_current_block;
                 // fall into the mutex release and restart below
-            }  else {
-                // really add an extension block
-                if (a->uw_extension_count == a->uw_max_extensions) { // can't extend further
+            } else { // potentially extend
+                // An object occupying 1/512th or more of an extension is separately allocated.
+                int oversized = nbytes > (sword_t)(a->uw_growth_amount >> 9);
+                long request = oversized ? nbytes : (sword_t)a->uw_growth_amount;
+                if (a->uw_length + request > a->uw_size_limit) { // limit reached
                     ARENA_MUTEX_RELEASE(a);
-                    lose("Fatal: arena memory exhausted");
+                    lose("Fatal: won't add arena %s block. Length=%lx request=%lx max=%lx",
+                         oversized ? "huge-object" : "extension",
+                         a->uw_length, request, a->uw_size_limit);
+                }
+                // For a huge object, ensure that there will be adequate space after aligning
+                // the bounds. Don't worry about it for a regular extension block though
+                // because there's no chance that the current request won't fit.
+                // Moreover, use malloc() for oversized objects instead of the macro
+                // because we don't actually remember the originally requested size
+                // to pass to ARENA_DISPOSE_MEMORY in arena_release_memblks
+                long actual_request;
+                char* new_mem;
+                if (oversized) {
+                    actual_request = request + 3*CHUNK_ALIGN;
+                    new_mem = malloc(actual_request);
+                } else {
+                    actual_request = request;
+                    new_mem = ARENA_GET_OS_MEMORY(actual_request);
                 }
-                char* new_mem = ARENA_GET_OS_MEMORY(a->uw_growth_amount);
                 if (new_mem == 0) {
                     ARENA_MUTEX_RELEASE(a);
                     lose("Fatal: arena memory exhausted and could not obtain more memory");
                 }
                 struct arena_memblk* extension= (void*)new_mem;
                 char* mem_base = new_mem + sizeof (struct arena_memblk);
+                // Alignment serves the needs of hide/unhide because mprotect()
+                // operates only on boundaries as dictated by the OS and/or CPU.
                 char* aligned_mem_base = PTR_ALIGN_UP(mem_base, CHUNK_ALIGN);
                 extension->freeptr = aligned_mem_base;
-                char* limit = new_mem + a->uw_growth_amount;
+                char* limit = new_mem + actual_request;
                 char* aligned_limit = PTR_ALIGN_DOWN(limit, CHUNK_ALIGN);
                 extension->limit = aligned_limit;
                 extension->next = NULL;
                 extension->padding = 0;
-                a->uw_length += a->uw_growth_amount; // tally up the total length
-                a->uw_extension_count++;
+                // tally up the total length
+                // For huge objects, count only the directly requested space,
+                // not the "bookends" (alignment chunks)
+                a->uw_length += request;
+                if (oversized) {
+                    long usable_space = aligned_limit - aligned_mem_base;
+                    // This should assert() because it's very bad, but assertions can be
+                    // disabled, so explicitly lose() if it happens.
+                    if (nbytes > usable_space) lose("alignment glitch");
+                    extension->next = (void*)a->uw_huge_objects;
+                    a->uw_huge_objects = (uword_t)extension;
+                    // A huge object needs to be zero-filled. It probably was, because it
+                    // probably just got mmapped() but there's no way to know.
+                    memset(aligned_mem_base, 0, nbytes);
+                    ARENA_MUTEX_RELEASE(a);
+                    return aligned_mem_base;
+                }
                 mem->next = extension;
                 // Other threads can start using the new block already
                 a->uw_current_block = (lispobj)extension;
@@ -294,11 +345,16 @@ lispobj* handle_arena_alloc(struct thread* th, struct alloc_region* region,
      *  1. amount remaining in the TLAB is not significant. Just toss it out.
      *     Refill the TLAB with the default quantum plus the user's actual request.
      *     Then carve out the new object from the beginning of the TLAB.
+     *     Exception: if 'nbytes' exceeds 64KiB then always treat it as case 2.
+     *     This avoids an anomaly in the situation where 'nbytes' so large that it
+     *     can not be obtained from the current arena block, but instead uses malloc()
+     *     directly. In such case, attempting to refill the TLAB from the same
+     *     block is futile.
      *
      *  2. amount remaining in the TLAB is worth keeping.
      *     Don't refill the TLAB; just make the new object as a discrete claim.
      */
-    if (avail < min_keep) { // case 1
+    if (avail < min_keep && nbytes <= 65536) { // case 1
         struct arena* in_use_arena = (void*)native_pointer(th->arena);
         __sync_fetch_and_add(&in_use_arena->uw_bytes_wasted, avail);
         long total_request = nbytes + 8192;
@@ -491,10 +547,42 @@ int find_dynspace_to_arena_ptrs(lispobj arena, lispobj result_buffer)
     return result;
 }
 
+static int count_arena_objects(lispobj arena, int *pnchunks)
+{
+    struct arena* a = (void*)native_pointer(arena);
+    struct arena_memblk* blk = (void*)a->uw_first_block;
+    int nobjects = 0;
+    int nchunks = 0;
+    do {
+        ++nchunks;
+        lispobj* where = (void*)ALIGN_UP(((uword_t)blk + sizeof (struct arena_memblk)),
+                                         CHUNK_ALIGN);
+        lispobj* limit = (void*)blk->freeptr;
+        while (where < limit) {
+            if (*where == (uword_t)-1) { // filler
+                where += 2;
+            } else {
+                ++nobjects;
+                where += object_size(where);
+            }
+        }
+        blk = blk->next;
+    } while (blk);
+    *pnchunks = nchunks;
+    return nobjects;
+}
+
 void arena_mprotect(lispobj arena, int option)
 {
 #ifndef LISP_FEATURE_WIN32
-    int prot = option ? PROT_NONE : (PROT_READ|PROT_WRITE|PROT_EXEC);
+    if (option) { // protecting
+        int nchunks;
+        int count = count_arena_objects(arena, &nchunks); // mainly as a sanity-check
+        fprintf(stderr, "arena_mprotect %p: %d objects in %d chunk(s)\n",
+                (void*)arena, count, nchunks);
+    }
+    // PROT_EXEC is not needed. Code blobs always go to dynamic or immobile space
+    int prot = option ? PROT_NONE : (PROT_READ|PROT_WRITE /*|PROT_EXEC*/);
     struct arena* a = (void*)native_pointer(arena);
     struct arena_memblk* blk = (void*)a->uw_first_block;
     do {
@@ -504,6 +592,7 @@ void arena_mprotect(lispobj arena, int option)
         // the block itself is not within [base,limit] and so can be read even if prot==PROT_NONE
         blk = blk->next;
     } while (blk);
+    // TODO: mprotect huge-object blocks
 #endif
 }
 
@@ -514,8 +603,8 @@ lispobj arena_find_containing_object(lispobj arena, char* ptr)
     do {
         lispobj* where = (void*)ALIGN_UP(((uword_t)blk + sizeof (struct arena_memblk)),
                                          CHUNK_ALIGN);
-        lispobj* limit = (void*)blk->limit;
-        while (where <  limit) {
+        lispobj* limit = (void*)blk->freeptr;
+        while (where < limit) {
             if (*where == (uword_t)-1) { // filler
                 where += 2;
             } else {
@@ -533,20 +622,40 @@ lispobj arena_find_containing_object(lispobj arena, char* ptr)
 int diagnose_arena_fault(os_context_t* context, char *addr)
 {
 #ifndef LISP_FEATURE_WIN32
+    if (!arena_chain) return 0; // not handled
     lispobj arena = find_containing_arena((lispobj)addr);
-    if (!arena) return 0; // not handled
-    if (arena && ((struct arena*)native_pointer(arena))->hidden == LISP_T) {
-        arena_mprotect(arena, 0); // unprotect it and find the object
-        lispobj obj = arena_find_containing_object(arena, addr);
-        if (obj) {
-            fprintf(stderr, "access @ %p sees hidden arena object @ %p in arena %p\n",
-                    addr, (void*)obj, (void*)arena);
-            fflush(stderr);
-        }
-        //        arena_mprotect(arena, 1); // put it back the way it was
-        lisp_memory_fault_error(context, addr);
-        return 1;
+    struct thread* th = get_sb_vm_thread();
+    struct thread_instance* instance = (void*)native_pointer(th->lisp_thread);
+    lispobj name = instance->name;
+    char *c_string = 0;
+    if (name != NIL) {
+        struct vector* string = (void*)native_pointer(name);
+        if (header_widetag(string->header) == SIMPLE_BASE_STRING_WIDETAG)
+            c_string = (char*)string->data;
     }
-    return 0;
+    fprintf(stderr, "trying diagnose_arena_fault(%p,'%s')\n",
+            addr, c_string);
+    if (!arena) {
+        fprintf(stderr, "fault is not in an arena\n");
+        fflush(stderr);
+        return 0; // not handled
+    }
+    int hidden = ((struct arena*)native_pointer(arena))->hidden == LISP_T;
+    fprintf(stderr, "fault in arena %p [%s]\n", (void*)arena, (hidden ? "HIDDEN" : "visible"));
+    fflush(stderr);
+    if (!hidden) return 0;
+    arena_mprotect(arena, 0); // unprotect it and find the object
+    fprintf(stderr, "unprotected OK\n");
+    fflush(stderr);
+    lispobj obj = arena_find_containing_object(arena, addr);
+    if (!obj) {
+        fprintf(stderr, "could not find containing lispobj\n");
+        fflush(stderr);
+        return 0;
+    }
+    fprintf(stderr, "access of object @ %p\n", (void*)obj);
+    fflush(stderr);
+    lisp_memory_fault_error(context, addr);
 #endif
+    return 1;
 }
